#========================================================================================
# package      Perl::Critic::Policy::DE_EPAGES::Modules::RequireUtf8BOM
# state        public
#----------------------------------------------------------------------------------------
# description  This policy checks the character encoding of the file.
#               Perl files should genrally be saved as iso-8859-1 (Latin1).
#               Only if the file contains text with international characters, it may be
#               useful to use utf-8. In this case a by order mark (BOM) is required to
#               recognize the character set.
#========================================================================================
package Perl::Critic::Policy::DE_EPAGES::Modules::RequireUtf8BOM;
use base 'Perl::Critic::Policy';

use strict;
use warnings;
use Perl::Critic::Utils qw{ :severities };

use DE_EPAGES::Core::API::File qw( DetectFileEncoding GetFileContent );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );

#-----------------------------------------------------------------------------

sub supported_parameters { return qw() }
sub default_severity     { return $SEVERITY_HIGHEST }
sub default_themes       { return qw( bugs epages ) }
sub applies_to           { return 'PPI::Document' }

#-----------------------------------------------------------------------------
sub violates {
    my $self = shift;
    my ( $elem, $doc ) = @_;

    my $Filename = $doc->filename or return;

    my $Encoding = DetectFileEncoding( $Filename );
    my $PerlContent = ${GetFileContent( $Filename )};

    if( defined $Encoding and $Encoding eq 'utf-8' ) {
        if( $PerlContent !~ /use\s+utf8/ ) {
            return $self->violation( 'UTF-8 file does not contain "use '.'utf8;"',
                'Consider saving the files as iso-8859-1 (Latin1) or add "use '.'utf8;".',
                $elem );
        }
        if( $PerlContent =~ /[ #]\x{00a7}/ ) {
            return $self->violation( 'File has utf-8 BOM, but appears to contain iso-8859-1 content',
                'Save the files as iso-8859-1 (Latin1).',
                $elem );
        }
    }

    if ( isValid($Encoding) and $Encoding ne 'utf-8' and $Encoding ne 'iso-8859-1' ) {
        return $self->violation( 'Unsupported character encoding',
            "Detected encoding is: '$Encoding'. Save the files as iso-8859-1 or utf-8.",
            $elem );
    }

    return;
}

1;
