use strict;
use version;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Config qw( DatabaseConnectionData );
use DE_EPAGES::Database::API::DBImysql qw( hasGrantPrivilege databaseVersion );
use DE_EPAGES::Database::API::Connection qw(RunOnStore GetCurrentDBHandle);

sub Main {

    local $| = 1;

    my $Password;
    my $ConnectionName;
    my $StoreName;
    my $DBName;
    my $DBLogin;
    my $DBPassword;
    my $Help;
    my $Verbose;

    GetOptions(
        'connection=s' => \$ConnectionName,
        'storename=s'  => \$StoreName,
        'passwd=s'     => \$Password,
        'dbname=s'     => \$DBName,
        'dblogin=s'    => \$DBLogin,
        'dbpasswd=s'   => \$DBPassword,
        'help'         => \$Help,
        'verbose'      => \$Verbose,
    );

    usage() if $Help;

    unless ($StoreName) {
        print STDERR "Missing parameter -storename\n\n";
        usage();
    }

    #start time
    my $time1 = time();

    # throws an error if StoreName does not exist in Database.d/$Store.conf
    $ConnectionName = DatabaseConnectionData($StoreName)->{'Backup'} unless $ConnectionName;
    $ConnectionName = 'Backup' unless $ConnectionName;
    print "using connection $ConnectionName\n" if $Verbose;

    RunOnStore (
        Store    => $ConnectionName,
        DBPassword => $DBPassword,
        Sub      => sub {
            my $dbi = GetCurrentDBHandle();
            Error('NO_GRANT_PRIV') unless $dbi->hasGrantPrivilege();

            # Read login data from conf file if not set via command line.
            my $hSection = DatabaseConnectionData($StoreName);
            $DBLogin    = $hSection->{'login'}    unless defined $DBLogin;
            $DBPassword = $hSection->{'passwd'}   unless defined $DBPassword;
            $DBName     = $hSection->{'Database'} unless defined $DBName;

            my $DBVersion = $dbi->databaseVersion;

            # create a user if the login does not exist
            my $aRows = $dbi->execute( 'SELECT 1 FROM mysql.user WHERE user = ? AND host="localhost"', [$DBLogin] );
            if ( 1 == scalar @$aRows ) {
                $aRows = $dbi->execute( 'SELECT 1 FROM mysql.user WHERE user = ? AND host="%"', [$DBLogin] );
                if ( 0 == scalar @$aRows ) {
                    if (version->parse($DBVersion) >= version->parse(8)) {
                        $dbi->do( "CREATE USER '$DBLogin' IDENTIFIED WITH mysql_native_password" );
                        $dbi->do( "SET PASSWORD FOR '$DBLogin' = '$DBPassword'" );
                    } else {
                        $dbi->do( "CREATE USER '$DBLogin' IDENTIFIED BY '$DBPassword'" );
                    }
                    $dbi->do( "GRANT ALL ON $DBName.* TO '$DBLogin'" );
                    $dbi->do( 'FLUSH PRIVILEGES' );
                }

                #alter views for storedb
                if($DBName eq 'storedb') {
                    #alter view for v_forumthread
                    $dbi->execute("ALTER ALGORITHM=UNDEFINED DEFINER=`$DBLogin`@`%` SQL SECURITY DEFINER VIEW `$DBName`.`v_forumthread` AS "
                                  . _ViewStatement('v_forumthread'));

                    #alter view for v_visibleforumthread
                    $dbi->execute("ALTER ALGORITHM=UNDEFINED DEFINER=`$DBLogin`@`%` SQL SECURITY DEFINER VIEW `$DBName`.`v_visibleforumthread` AS "
                                  . _ViewStatement('v_visibleforumthread'));
                }
                #alter views for aspooldb
                elsif($DBName eq 'aspooldb') {
                    #alter view v_database
                    $dbi->execute("ALTER ALGORITHM=UNDEFINED DEFINER=`$DBLogin`\@`%` SQL SECURITY DEFINER VIEW `$DBName`.`v_database` AS "
                                  . _ViewStatement('v_database'));

                    #alter view v_lookup
                    $dbi->execute("ALTER ALGORITHM=UNDEFINED DEFINER=`$DBLogin`\@`%` SQL SECURITY DEFINER VIEW `$DBName`.`v_lookup` AS "
                                  . _ViewStatement('v_lookup'));

                    #alter view v_lookup_host
                    $dbi->execute("ALTER ALGORITHM=UNDEFINED DEFINER=`$DBLogin`\@`%` SQL SECURITY DEFINER VIEW `$DBName`.`v_lookup_host` AS "
                                  . _ViewStatement('v_lookup_host'));
                }

                #delete 'user'@'localhost'
                $dbi->do( "DROP USER '$DBLogin'\@'localhost' " );
                $dbi->do( 'FLUSH PRIVILEGES' );
            }

        }

    );

    my $time2 = time();
    my $seconds = $time2 - $time1;
    printf "need %.3f seconds\n", $seconds if $Verbose;
}

sub _ViewStatement {
    my ($View) = @_;

    my ($Select, $As, $From, $Where, $GroupBy);

    if($View =~ /forumthread/) {
        $Select = 'select `ft`.`forumid`';
        $As = 'AS `forumid`,`ft`.`forumpostid` AS `forumpostid`,`ft`.`subject` AS `subject`,max(`fp`.`forumpostid`)'
            .' AS `lastpostid`,count(`fp`.`forumpostid`) AS `postcount`';
        $From = 'from (`storedb`.`forumpost` `ft` join `storedb`.`forumpost` `fp` on((`fp`.`threadid` = `ft`.`forumpostid`)))';
        $Where = $View eq 'v_forumthread' ? 'where (`ft`.`forumpostid` = `ft`.`threadid`)' :
            'where ((`ft`.`forumpostid` = `ft`.`threadid`) and (`ft`.`isvisible` = 1) and (`fp`.`isvisible` = 1))';
        $GroupBy = 'group by `ft`.`forumid`,`ft`.`forumpostid`,`ft`.`subject`';
    } elsif ($View eq 'v_database') {
        $Select = 'select `p`.`name`';
        $As = 'AS `poolname`,`s`.`name` AS `databasename`';
        $From = 'from (`aspooldb`.`store` `s` join `aspooldb`.`pool` `p` on((`p`.`poolid` = `s`.`poolid`)))';
    } elsif ($View eq 'v_lookup') {
        $Select = 'select `h`.`name`';
        $As = 'AS `httphost`,`r`.`uri` AS `uri`,`p`.`name` AS `poolname`,`st`.`name` AS `storename`,`si`.`guid`'
            .' AS `siteguid`,`si`.`objectpath` AS `objectpath`,`si`.`maxservers` AS `sitemaxservers`';
        $From = 'from ((((`aspooldb`.`host` `h` join `aspooldb`.`request` `r` on((`r`.`hostid` = `h`.`hostid`)))'.
            ' join `aspooldb`.`site` `si` on((`si`.`siteid` = `r`.`siteid`))) join `aspooldb`.`store` `st`'.
            ' on((`st`.`storeid` = `si`.`storeid`))) join `aspooldb`.`pool` `p` on((`p`.`poolid` = `st`.`poolid`)))';
    } elsif ($View eq 'v_lookup_host') {
        $Select = 'select `h`.`name`';
        $As = 'AS `httphost`,`p`.`name` AS `poolname`,`st`.`name` AS `storename`,`si`.`guid`'
            .' AS `siteguid`,`si`.`objectpath` AS `objectpath`,`si`.`maxservers` AS `sitemaxservers`';
        $From = 'from (((`aspooldb`.`host` `h` join `aspooldb`.`site` `si` on((`si`.`siteid` = `h`.`siteid`)))'.
            ' join `aspooldb`.`store` `st` on((`st`.`storeid` = `si`.`storeid`))) join `aspooldb`.`pool` `p`'.
            ' on((`p`.`poolid` = `st`.`poolid`)))';
    }

    return "$Select $As $From $Where $GroupBy"; ##no critic qw(ProhibitVariablesInSQLStatements)
}

sub usage {
    print <<END_USAGE;
Description:
    Changes existing user\@localhost to user\@\%.
    Alters views for storedb and ASPoolDB.
    If the parameters -dbname and -dblogin are not specified, names are
    auto-generated from the -storename parameter, for example:
      -storename=Store007 -> -storedb=storedb007 -dbuser=usr_storedb007

Usage:
    perl $0 [options]

options:
    -connection    store name for master database                   (Backup)
    -passwd        database user password                         (optional)
    -storename     store name for the database connection         (required)
    -dbname        database name                                  (optional)
    -dblogin       database user alias                            (optional)
    -dbpasswd      database user password                         (optional)

Example:
    perl -storename Store2
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );

1;
