#========================================================================================
# package      DE_EPAGES::Installer::API::PatchDB
# state        public
#----------------------------------------------------------------------------------------
# description  Functions for managing the PatchDB.
#========================================================================================
package DE_EPAGES::Installer::API::PatchDB;
use base qw (Exporter);
use strict;
use version;

our @EXPORT_OK = qw (
    CreatePatchDB
    DropPatchDB
);

use DE_EPAGES::Database::API::Config qw( DatabaseConnectionData );
use DE_EPAGES::Database::API::Connection qw(RunOnStore GetCurrentDBHandle);
use DE_EPAGES::Database::API::DBImysql qw( databaseVersion );

#========================================================================================
# function     CreatePatchDB
# state        public
#----------------------------------------------------------------------------------------
# syntax       CreatePatchDB($hFlags);
#----------------------------------------------------------------------------------------
# description  creates patchsysteminfodb (unless exists)
#----------------------------------------------------------------------------------------
# input        $hFlags | flags
#                   <ul>
#                       <li>verbose - show progress
#                   </ul>
#               | ref.hash
#========================================================================================
sub CreatePatchDB {
    my ($hFlags) = @_;

    my $DBExists = "database already exists: %s\n";
    my $CreatingDB = "creating database: %s (login=%s, passwd=%s)\n";

    my $hConn = DatabaseConnectionData('PatchSystemInfoDB');
    RunOnStore(
        Store => $hConn->{Backup},
        Sub   => sub {
            my $dbi = GetCurrentDBHandle();
            if ($dbi->existsDatabase($hConn->{Database})) {
                printf($DBExists, $hConn->{Database}) if $hFlags->{verbose};
            } else {
                printf($CreatingDB, $hConn->{Database}, $hConn->{login},
                       $hConn->{passwd}) if $hFlags->{verbose};
                $dbi->createEpagesDatabase($hConn->{Database}, $hConn->{login},
                                           { 'DBPassword' => $hConn->{passwd} });
                # MySQL 5.8 does not use PASSWORD()
                my $HashedPswd = version->parse($dbi->databaseVersion) >= version->parse(8) ?
                    "'$hConn->{passwd}'" : "PASSWORD('$hConn->{passwd}')";
                # if user $hConn->{login} existed before createEpagesDatabase,
                # password might differ and must set to $hConn->{passwd}
                $dbi->do("SET PASSWORD FOR '$hConn->{login}'\@'localhost' = $HashedPswd");
                $dbi->do("SET PASSWORD FOR '$hConn->{login}' = $HashedPswd");
            }
        }
        );
}

#========================================================================================
# function     DropPatchDB
# state        public
#----------------------------------------------------------------------------------------
# syntax       DropPatchDB($hFlags);
#----------------------------------------------------------------------------------------
# description  drops patchsysteminfodb (if exists)
#----------------------------------------------------------------------------------------
# input        $hFlags | flags
#                   <ul>
#                       <li>verbose - show progress
#                   </ul>
#               | ref.hash
#========================================================================================
sub DropPatchDB {
    my ($hFlags) = @_;

    my $DBMissing = "database does not exist: %s\n";
    my $DroppingDB = "dropping database: %s\n";

    my $hConn = DatabaseConnectionData('PatchSystemInfoDB');
    RunOnStore(
        Store => $hConn->{Backup},
        Sub => sub {
            my $dbi = GetCurrentDBHandle();
            if (not $dbi->existsDatabase($hConn->{Database})) {
                printf($DBMissing, $hConn->{Database}) if $hFlags->{verbose};
            } else {
                printf($DroppingDB, $hConn->{Database}) if $hFlags->{verbose};
                $dbi->dropDatabase($hConn->{Database});
            }
        }
        );
}

1;
