#========================================================================================
# package      DE_EPAGES::MolliePayments::API::MollieOrderShipmentService
# state        private
#----------------------------------------------------------------------------------------
# description  Service module aimed to handle order shipment related processes
#========================================================================================
package DE_EPAGES::MolliePayments::API::MollieOrderShipmentService;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw (LogError);
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::MolliePayments::API::Constants qw (
    MOLLIE_ORDER_STATUS_CANCELED
    ORDER_PARAM_EMBED_PAYMENTS
    PAYMENT_TYPE_MOLLIE_PAYMENTS
);
use DE_EPAGES::MolliePayments::API::MolliePaymentsClient;
use DE_EPAGES::MolliePayments::API::PaymentStatusService;
use DE_EPAGES::MolliePayments::API::Requests::Orders;
use DE_EPAGES::MolliePayments::API::Requests::OrdersLines;
use DE_EPAGES::MolliePayments::API::Requests::OrdersShipments;
use DE_EPAGES::MolliePayments::API::Tools qw ( ParseErrorStringToHash );

use strict;

#ERRORDEFINITION
sub ErrorDefinition () {
    return {
        'UNRESOLVABLE_ORDER_MISMATCH' => 'order line item #LineItemGUID not found in Mollie order.'
    }
}
#ENDERRORDEFINITION

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::MolliePayments::API::OrdersServiceBuilder->new( $CustomerOrder );
#----------------------------------------------------------------------------------------
# description  instantiation method of the OrdersServiceBuilder object
#----------------------------------------------------------------------------------------
# input        $CustomerOrder | order object | DE_EPAGES::Order::API::Object::CustomerOrder
# return       $Object | new object | DE_EPAGES::MolliePayments::API::MollieOrderShipmentService
#========================================================================================
sub new {
    my $class = shift;
    my ($CustomerOrder) = @_;

    TestObject('CustomerOrder', $CustomerOrder, 'DE_EPAGES::Order::API::Object::CustomerOrder');
    my $LineItemContainer = $CustomerOrder->get('LineItemContainer');
    my $LineItemPayment = $LineItemContainer->get('Payment');
    TestObject(
        'LineItemPaymentMolliePayments',
        $LineItemPayment,
        'DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'
    );

    return bless {
        'order' => $CustomerOrder,
        'container' => $LineItemContainer,
        'lineItemPayment' => $LineItemPayment
    }, $class;
}

#========================================================================================
# function     updateMollieOrderLineItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = updateMollieOrderLineItems( );
#----------------------------------------------------------------------------------------
# description  update Mollie order line items to fit with epages order line items
#               in the event that they were modified.
#----------------------------------------------------------------------------------------
# return       $hResponse | status and message of this process | hash.ref
#========================================================================================
sub updateMollieOrderLineItems {
    my $self = shift;

    my $Client = $self->_buildClientToGetMollieOrder();
    my $hOrderResponse;
    eval {
        $hOrderResponse = $Client->doGetRequest();
    };
    if (ExistsError()) {
        return $self->_formatErrorResponseForDialog(GetError(), 'Error while gathering Mollie order');
    }
    LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, 'Mollie order gathering success', $hOrderResponse);

    my $TotalOrderAmount = $self->{'container'}->get('GrandTotal');
    my $AuthorizedAmount = $hOrderResponse->{'payments'}[0]->{'amount'}->{'value'};
    if ($AuthorizedAmount < $TotalOrderAmount) {
        return $self->_formatErrorResponseForDialog(
            {
                'title' => 'CaptureOverflow',
                'detail' => "Order amount ($TotalOrderAmount) exceeds authorized amount ($AuthorizedAmount)"
            },
            'Authorized amount exceeded'
        );
    }

    my $aMollieOrderLineItems = $hOrderResponse->{'lines'};
    my @OrderLineItemProductAliasNames = ();
    foreach my $EpOrderLineItem (@{$self->{'container'}->children()}) {
        my $MollieOrderLineItem = $self->_getMollieOrderLineItem($aMollieOrderLineItems, $EpOrderLineItem);

        if (defined($MollieOrderLineItem)) {
            if ($self->_isLineItemPriceMismatched($EpOrderLineItem, $MollieOrderLineItem)) {
                my $ClientToUpdate = $self->_buildClientToPutMollieOrderLineItem(
                        $EpOrderLineItem, $MollieOrderLineItem->{'id'}
                );
                eval {
                    $ClientToUpdate->doPutRequest();
                };
                if (ExistsError()) {
                    return $self->_formatErrorResponseForDialog(
                        GetError(),
                        "Error while updating Mollie order line item"
                    );
                }
            }
        } else {
            if ($EpOrderLineItem->get('LineItemPrice') != 0) {
                Error('UNRESOLVABLE_ORDER_MISMATCH',{ 'LineItemGUID' => $EpOrderLineItem->guid() });
            }
        }
        if ($EpOrderLineItem->instanceOf('LineItemProduct')) {
            push(@OrderLineItemProductAliasNames, $EpOrderLineItem->alias());
        }
    }

    my $aMollieLineItemIds = $self->_getMollieLineItemsToRemove(
            $aMollieOrderLineItems, \@OrderLineItemProductAliasNames
    );
    if (scalar @$aMollieLineItemIds > 0) {
        my $ClientToDelete = $self->_buildClientToDeleteMollieOrderLineItems($aMollieLineItemIds);
        eval {
            $ClientToDelete->doDeleteRequest();
        };
        if (ExistsError()) {
            return $self->_formatErrorResponseForDialog(GetError(), "Error while deleting Mollie order line item(s)");
        }
    }

    return {
        'StatusCode' => 'OK',
        'Message' => 'Success'
    }

}

#========================================================================================
# function     shipMollieOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = shipMollieOrder( );
#----------------------------------------------------------------------------------------
# description  ship mollie order and automatically capture payment
#----------------------------------------------------------------------------------------
# return       $hResponse | status and message of this process | hash.ref
#========================================================================================
sub shipMollieOrder {
    my $self = shift;

    my $Client = $self->_buildClientToShipMollieOrder();

    my $hResponse;
    eval {
        $hResponse = $Client->doPostRequest()
    };
    if (ExistsError()) {
        return $self->_formatErrorResponseForDialog(GetError(), "Error while shipping Mollie order");
    }
    my $AmountCaptured = $self->{'container'}->get('GrandTotal');
    $self->{'lineItemPayment'}->set({ 'Amount' => $AmountCaptured });
    $self->_updateLineItemPaymentStatus($self->{'lineItemPayment'});

    my $SuccessMessage = 'Shipment Id ' . $hResponse->{'id'} . ' created';

    return {
        'StatusCode' => 'OK',
        'Message'    => $SuccessMessage
    };
}

#----------------------------------------------------------------------------------------
sub _buildClientToGetMollieOrder {
    my $self = shift;

    my $OrderRequest =  DE_EPAGES::MolliePayments::API::Requests::Orders->new({
        'payment' => $self->{'lineItemPayment'}->get('PaymentMethod'),
    });
    $OrderRequest->withOrderId($self->{'lineItemPayment'}->get('MollieOrderId'));
    $OrderRequest->withEmbed(ORDER_PARAM_EMBED_PAYMENTS);

    LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, 'Gather Mollie Order', $OrderRequest->getQueryParams('GET'));

    my $Client = DE_EPAGES::MolliePayments::API::MolliePaymentsClient->new($OrderRequest);

    return $Client->withTestMode($self->{'lineItemPayment'}->get('IsTestTransaction'));
}

#----------------------------------------------------------------------------------------
sub _buildClientToShipMollieOrder {
    my $self = shift;

    my $OrderRequest =  DE_EPAGES::MolliePayments::API::Requests::OrdersShipments->new({
        'container' => $self->{'container'},
        'payment' => $self->{'lineItemPayment'}->get('PaymentMethod'),
    });

    LogPayment(
        PAYMENT_TYPE_MOLLIE_PAYMENTS,
        'Ship Mollie order and capture payment',
        $OrderRequest->getPayload('POST')
    );

    return DE_EPAGES::MolliePayments::API::MolliePaymentsClient->new($OrderRequest);
}

#----------------------------------------------------------------------------------------
sub _formatErrorResponseForDialog {
    my $self = shift;
    my ($Error, $LogTitleMessage) = @_;

    my $hErrorResponse;
    if (ref($Error) eq 'HASH') {
        $hErrorResponse = {
            'content' => $Error
        }
    } else {
        LogError($Error);
        $hErrorResponse = ParseErrorStringToHash($Error->vars->{'Response'}, $Error->vars->{'Status'});
    }

    LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, $LogTitleMessage, $hErrorResponse);

    my $ErrorTitle;
    my $ErrorMessage;
    if (defined($hErrorResponse->{'content'})) {
        $ErrorTitle = $hErrorResponse->{'content'}->{'title'};
        $ErrorMessage = $hErrorResponse->{'content'}->{'detail'};
    }

    return {
        'StatusCode' => $ErrorTitle,
        'Message' => $ErrorMessage
    };
}

#----------------------------------------------------------------------------------------
sub _getMollieOrderLineItem {
    my $self = shift;
    my ($aMollieOrderLineItems, $EpOrderLineItem) = @_;

    my $EpOrderLineItemAlias = $EpOrderLineItem->alias();
    my $IsLineItemShipping = $EpOrderLineItem->instanceOf('LineItemShipping');

    foreach my $MollieOrderLineItem (@$aMollieOrderLineItems) {
        if ($MollieOrderLineItem->{'metadata'}->{'alias'} eq $EpOrderLineItemAlias) {
            return $MollieOrderLineItem;
        } elsif ($IsLineItemShipping && ($MollieOrderLineItem->{'metadata'}->{'alias'} =~ m/LineItemShipping/)) {
            return $MollieOrderLineItem;
        }
    }

    return;
}

#----------------------------------------------------------------------------------------
sub _isLineItemPriceMismatched {
    my $self = shift;
    my ($EpOrderLineItem, $MollieOrderLineItem) = @_;

    return 1 unless ($EpOrderLineItem->get('LineItemPrice') == $MollieOrderLineItem->{'totalAmount'}->{'value'});
    # don't check quantity and base price because they may not match since Mollie order generation
    # (i.e. rounded quantities or unary quantities for epages line items without quantity)
    return 0;
}

#----------------------------------------------------------------------------------------
sub _buildClientToPutMollieOrderLineItem {
    my $self = shift;
    my ($EpOrderLineItem, $MollieOrderLineItemId) = @_;

    my $OrderLineRequest =  DE_EPAGES::MolliePayments::API::Requests::OrdersLines->new({
        'container' => $self->{'container'},
        'payment' => $self->{'lineItemPayment'}->get('PaymentMethod')
    });
    $OrderLineRequest->withOrderLineItem($EpOrderLineItem, $MollieOrderLineItemId);

    LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, 'Update Mollie order line item', $OrderLineRequest->getPayload('PUT'));
    my $Client = DE_EPAGES::MolliePayments::API::MolliePaymentsClient->new($OrderLineRequest);

    return $Client->withTestMode($self->{'lineItemPayment'}->get('IsTestTransaction'));
}

#----------------------------------------------------------------------------------------
sub _getMollieLineItemsToRemove {
    my $self = shift;
    my ($aMollieOrderLineItems, $aOrderLineItemProductAliasNames) = @_;

    my @MollieOrderLineItemIds = ();
    foreach my $MollieOrderLineItem (@$aMollieOrderLineItems) {
        next unless ($MollieOrderLineItem->{'metadata'}->{'alias'} =~ m/LineItemProduct/);
        next if ($MollieOrderLineItem->{status} eq MOLLIE_ORDER_STATUS_CANCELED);

        push(@MollieOrderLineItemIds, $MollieOrderLineItem->{'id'})
            unless ($MollieOrderLineItem->{'metadata'}->{'alias'} ~~ $aOrderLineItemProductAliasNames);
    }
    return \@MollieOrderLineItemIds;
}

#----------------------------------------------------------------------------------------
sub _buildClientToDeleteMollieOrderLineItems {
    my $self = shift;
    my ($aMollieLineItemIds) = @_;

    my $OrderLineRequest =  DE_EPAGES::MolliePayments::API::Requests::OrdersLines->new({
        'container' => $self->{'container'},
        'payment' => $self->{'lineItemPayment'}->get('PaymentMethod')
    });
    $OrderLineRequest->withOrderLineItemIdList($aMollieLineItemIds);

    LogPayment(
        PAYMENT_TYPE_MOLLIE_PAYMENTS,
        'Delete Mollie order line items',
        $OrderLineRequest->getQueryParams('DELETE')
    );
    my $Client = DE_EPAGES::MolliePayments::API::MolliePaymentsClient->new($OrderLineRequest);

    return $Client->withTestMode($self->{'lineItemPayment'}->get('IsTestTransaction'));
}

#----------------------------------------------------------------------------------------
sub _updateLineItemPaymentStatus {
    my $self = shift;
    my ($LineItemPayment) = @_;

    my $StatusService = DE_EPAGES::MolliePayments::API::PaymentStatusService->new();
    $StatusService->getMollieOrderAndHandleStatus($LineItemPayment);

    return;
}

1;
