#========================================================================================
# package      DE_EPAGES::MolliePayments::API::MolliePaymentsRefundsService
# state        public
# description  Refunds Service for MolliePayments
#========================================================================================
package DE_EPAGES::MolliePayments::API::MolliePaymentsRefundsService;

use DE_EPAGES::MolliePayments::API::MolliePaymentsClient;
use DE_EPAGES::MolliePayments::API::Requests::Payments;
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::MolliePayments::API::Constants qw ( PAYMENT_TYPE_MOLLIE_PAYMENTS );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::MolliePayments::API::Tools qw ( FormatFloatForMollie );

use strict;

use constant PAYMENT_STATUS_PAID => 'paid';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Mollie::API::MolliePaymentsRefundsService->new( %args );
#----------------------------------------------------------------------------------------
# description  Instantiates a new MolliePaymentsRefundsService.
#----------------------------------------------------------------------------------------
# input        $RequestObject | request object to handle | object
# return       MolliePaymentsRefundsService | object | DE_EPAGES::MolliePayments::API::MolliePaymentsRefundsService
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     updateRefundAmountsByGetMolliePayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentRefundsService->updateRefundAmountsByGetMolliePayment( $LineItemPayment );
#----------------------------------------------------------------------------------------
# description  Requests Mollie Payment information and updates the LineItemPayment with refunds info received.
#----------------------------------------------------------------------------------------
# input        LineItemPayment | LineItemPayment | object
#========================================================================================
sub updateRefundAmountsByGetMolliePayment {
    my $self = shift;
    my ($LineItemPayment) = @_;

    $self->_validateLineItemPayment($LineItemPayment);
    $self->_validateOrder($LineItemPayment);

    my $hPaymentResponse = $self->_getMolliePayment($LineItemPayment, $LineItemPayment->get('PaymentMethod'));
    $self->updateLineItemRefundAmounts($LineItemPayment, $hPaymentResponse);

    return;
}

#========================================================================================
# function     doRefund
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentRefundsService->doRefund( $LineItemPayment, $RefundAmount );
#----------------------------------------------------------------------------------------
# description  Creates a Mollie Payment Refund and updates the LineItemPayment with refunds info received.
#----------------------------------------------------------------------------------------
# input        LineItemPayment | LineItemPayment | object
# input        RefundAmount | RefundAmount | float
# return       updated Refunds info | HashRef | HashRef
#========================================================================================
sub doRefund {
    my $self = shift;
    my ($LineItemPayment, $RefundAmount) = @_;

    $self->_validateLineItemPayment($LineItemPayment);
    $self->_validateOrder($LineItemPayment);

    my $MolliePaymentMethod = $LineItemPayment->get('PaymentMethod');
    my $CurrencyId = $LineItemPayment->get('CurrencyID');
    my $MolliePaymentId = $LineItemPayment->get('TransID');

    $self->_LogPayment(
            'Request to Create Refund',
            {
                'RefundAmount' => $RefundAmount,
                'CurrencyID' => $CurrencyId,
                'PaymentId' => $MolliePaymentId
            }
    );

    my $hPaymentsData = {
        'payment' => $MolliePaymentMethod
    };
    my $PaymentsRequest = $self->_createPaymentsRequest($hPaymentsData);
    $PaymentsRequest
        ->withRefundAmountAndCurrencyId($RefundAmount, $CurrencyId)
        ->withPaymentId($MolliePaymentId);

    my $MolliePaymentsClient = $self->_createMolliePaymentsClient($PaymentsRequest);
    my $hRefundResponse = $MolliePaymentsClient->withTestMode($LineItemPayment->get('IsTestTransaction'))->doPostRequest();

    $self->_LogPayment(
            'Create Refund Response',
            {
                'Response' => $hRefundResponse,
                'PaymentId' => $MolliePaymentId
            }
    );

    my $hPaymentResponse = $self->_getMolliePayment($LineItemPayment, $MolliePaymentMethod);
    my $hUpdatedRefundAmounts = $self->updateLineItemRefundAmounts($LineItemPayment, $hPaymentResponse);

    return {
        'paymentId' => $MolliePaymentId,
        'id' => $hRefundResponse->{'id'},
        'status' => $hRefundResponse->{'status'},
        'amount' => $hRefundResponse->{'amount'},
        'updatedRefundAmounts' => $hUpdatedRefundAmounts
    }
}

#========================================================================================
# function     updateLineItemRefundAmounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentRefundsService->updateLineItemRefundAmounts( $LineItemPayment, $hPaymentResponse );
#----------------------------------------------------------------------------------------
# description  Updates the LineItemPayment with refunds info received on a payment response.
#----------------------------------------------------------------------------------------
# input        LineItemPayment | LineItemPayment | object
# input        hPaymentResponse | hPaymentResponse | HashRef
# return       updated Refunds info | HashRef | HashRef
#========================================================================================
sub updateLineItemRefundAmounts {
    my $self = shift;
    my ($LineItemPayment, $hPaymentResponse) = @_;

    $self->_validateLineItemPayment($LineItemPayment);

    my $MolliePaymentId = $LineItemPayment->get('TransID');

    # an order's payment only supports refunds if amountRemaining is present in Mollie response
    # this amountRemaining can be 25euros more than the payment amount
    # we only want to refund until the original payment amount

    my $hUpdatedRefundAmounts = {};
    my $OriginalPaymentAmount = $LineItemPayment->get('Amount');

    if (isValid($hPaymentResponse->{'amountRemaining'})) {
        $LineItemPayment->set({'AmountRemaining' => $OriginalPaymentAmount});
        $hUpdatedRefundAmounts->{'AmountRemaining'} = $OriginalPaymentAmount;

        if (isValid($hPaymentResponse->{'amountRefunded'})) {
            my $AmountRefunded = $hPaymentResponse->{'amountRefunded'}->{'value'};

            my $AmountRemaining = FormatFloatForMollie($OriginalPaymentAmount - $AmountRefunded);

            if ($AmountRemaining < 0) {
                $AmountRemaining = FormatFloatForMollie(0);
            }

            $LineItemPayment->set({'AmountRefunded' => $AmountRefunded});
            $hUpdatedRefundAmounts->{'AmountRefunded'} = $AmountRefunded;

            $LineItemPayment->set({'AmountRemaining' => $AmountRemaining});
            $hUpdatedRefundAmounts->{'AmountRemaining'} = $AmountRemaining;
        }
    }

    $self->_LogPayment(
            'Updated LineItemPayment Refund Amounts',
            {
                'UpdatedRefundAmounts' => $hUpdatedRefundAmounts,
                'PaymentId' => $MolliePaymentId,
                'PaymentResponse' => $hPaymentResponse 
            }
        );

    return $hUpdatedRefundAmounts;
}

#----------------------------------------------------------------------------------------
# Private methods
#----------------------------------------------------------------------------------------
sub _validateLineItemPayment {
    my $self = shift;
    my ($LineItemPayment) = @_;

    TestObject('LineItemPaymentMolliePayments', $LineItemPayment, 'DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments');

    if ($LineItemPayment->get('TransStatus') ne PAYMENT_STATUS_PAID) {
        Error('To refund a paid status is required');
    }
}

sub _validateOrder {
    my $self = shift;
    my ($LineItemPayment) = @_;

    my $Order = $LineItemPayment->container()->parent();

    TestObject('Order', $Order, 'DE_EPAGES::Order::API::Object::CustomerOrder');
}

sub _getMolliePayment {
    my $self = shift;
    my ($LineItemPayment, $MolliePaymentMethod) = @_;

    my $MolliePaymentId = $LineItemPayment->get('TransID');
    my $hPaymentsData = {
        'payment' => $MolliePaymentMethod
    };
    my $PaymentsRequest = $self->_createPaymentsRequest($hPaymentsData);
    my $MolliePaymentsClient = $self->_createMolliePaymentsClient($PaymentsRequest);

    $self->_LogPayment(
        'Request to Get Payment',
        {'PaymentId' => $MolliePaymentId}
    );

    $PaymentsRequest->withPaymentId($MolliePaymentId);
    my $hPaymentResponse = $MolliePaymentsClient->withTestMode($LineItemPayment->get('IsTestTransaction'))->doGetRequest();

    $self->_LogPayment(
        'Get Payment Response',
        {
            'Response' => $hPaymentResponse,
            'PaymentId' => $MolliePaymentId
        }
    );
    return $hPaymentResponse;
}

sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, $Name, $DebugData, $Section, $Filter);
}
#----------------------------------------------------------------------------------------
sub _createMolliePaymentsClient() {
    my $self = shift;
    my ($Request) = @_;

    my $MolliePaymentsClient = DE_EPAGES::MolliePayments::API::MolliePaymentsClient->new($Request);
    return $MolliePaymentsClient;
}
#----------------------------------------------------------------------------------------
sub _createPaymentsRequest() {
    my $self = shift;
    my ($hData) = @_;

    my $Request = DE_EPAGES::MolliePayments::API::Requests::Payments->new($hData);
    return $Request;
}

1;

