#========================================================================================
# package      DE_EPAGES::MolliePayments::API::MolliePaymentsUrlBuilder
# state        public
#----------------------------------------------------------------------------------------
# description  Build return URL for Backlink
#========================================================================================
package DE_EPAGES::MolliePayments::API::MolliePaymentsUrlBuilder;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Url qw ( ParseQueryString );
use URI;
use URI::URL;
use DE_EPAGES::Object::API::Factory qw(LoadRootObject);
use DE_EPAGES::MolliePayments::API::Constants qw ( APP_SCOPES );

use strict;

use constant MOLLIE_V2_AUTHORIZE_CONNECT_URL => 'https://www.mollie.com/oauth2/authorize';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::MolliePayments::API::MolliePaymentsUrlBuilder
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     buildConnectAndReconnectUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildConnectAndReconnectUrl( $StateURL );
#----------------------------------------------------------------------------------------
# description  Build the connect URL used for on-boarding
#----------------------------------------------------------------------------------------
# input        $StateURL | Current URL in the MBO  | string
# return       $URL->as_string() | Mollie connect url | string
#========================================================================================
sub buildConnectAndReconnectUrl {
    my $self = shift;
    my ($StateURL) = @_;

    my $hQueryParameters = $self->_buildQueryParameters($StateURL);

    my $Url = $self->_buildEncodedUrl($hQueryParameters);

    return $Url;
}

#========================================================================================
# function     buildRedirectToMboUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildRedirectToMboUrl( $hQueryParameters );
#----------------------------------------------------------------------------------------
# description  Build redirect to MBO URL using returned data from MolliePayments
#----------------------------------------------------------------------------------------
# input        $hQueryParameters | Hashref which is returned from MolliePayments | hashref
# return       $URL->as_string() | BacklinkUrl with AuthorizationCode or Error | string
#========================================================================================
sub buildRedirectToMboUrl {
    my $self = shift;
    my ($hQueryParameters) = @_;
    my $aBacklinkUrl = $hQueryParameters->{'state'};

    if ( (!isValid($aBacklinkUrl)) or (!isValid($aBacklinkUrl->[0])) ) {
        Error('NoURLFound');
    }

    return $self->_urlBuilder($hQueryParameters);
}

#----------------------------------------------------------------------------------------
sub _buildEncodedUrl {
    my $self = shift;
    my ($hQueryParameters) = @_;

    my $Url = URI::URL->new(MOLLIE_V2_AUTHORIZE_CONNECT_URL);
    $Url->query_form($hQueryParameters);
    my $hBacklinkUrlParameters = ParseQueryString($Url->equery(), 'utf-8');
    $Url->query_form( $hBacklinkUrlParameters );

    return $Url;
}
#----------------------------------------------------------------------------------------
sub _buildQueryParameters {
    my $self = shift;
    my ($StateURL) = @_;

    my $System = LoadRootObject();
    my $ClientID = $System->get('MolliePayments_AppClientID');
    my $RedirectUrl = $System->get('MolliePayments_AppRedirectURL');

    return {
        'response_type'   => 'code',
        'client_id'       => $ClientID,
        'scope'           => join(' ', APP_SCOPES),
        'redirect_uri'    => $RedirectUrl,
        'state'           => $StateURL,
        'approval_prompt' => 'force',
    };
}

#----------------------------------------------------------------------------------------
sub _urlBuilder {
    my $self = shift;
    my ($hQueryParameters) = @_;

    # Original Change action must be removed
    delete($hQueryParameters->{'ChangeAction'});

    # State contains the URL which is backlink to MBO. No need to send it with QueryString again
    my $BackLinkUrl = delete($hQueryParameters->{'state'});

    my $URL = URI->new($BackLinkUrl->[0]);
    my $hBacklinkUrlParameters = ParseQueryString($URL->query(), 'utf-8');
    my $hQueryParam = {%{$hBacklinkUrlParameters}, %{$hQueryParameters}};
    $URL->query_form($hQueryParam);

    return $URL->as_string();
}

1;