#========================================================================================
# package      DE_EPAGES::MolliePayments::API::PaymentStatusService
# state        public
# description  Service for Updating Payment Status
#========================================================================================
package DE_EPAGES::MolliePayments::API::PaymentStatusService;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid GetCurrentDateTime );
use DE_EPAGES::MolliePayments::API::MolliePaymentsRefundsService;
use DE_EPAGES::MolliePayments::API::Requests::Orders;
use DE_EPAGES::MolliePayments::API::MolliePaymentsClient;
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::MolliePayments::API::Constants qw (
    LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS
    PAYMENT_METHOD_MOLLIE_PAYMENTS
    ORDER_PARAM_EMBED_PAYMENTS
    DEFINITIVE_MOLLIE_PAYMENT_STATUS
    MOLLIE_PAYMENT_STATUS_ERRORS
    SUPPORTED_MOLLIE_PAYMENT_STATUS
    PAYMENT_STATUS_PAID
    PAYMENT_STATUS_EXPIRED
    PAYMENT_TYPE_MOLLIE_PAYMENTS
    MOLLIE_ORDER_STATUS_CANCELED
);
use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::MolliePayments::API::PaymentStatusService->new( %args );
#----------------------------------------------------------------------------------------
# description  Instantiates a new PaymentStatusService with Epagesj Rest client set.
#----------------------------------------------------------------------------------------
# input        $RequestObject | request object to handle | object
# return       MolliePaymentsClient | object | DE_EPAGES::MolliePayments::API::MolliePaymentsClient
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;

    return $self;
}

#========================================================================================
# function     handleStatusOnPaymentWebhook
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::MolliePayments::API::PaymentStatusService->handleStatusOnPaymentWebhook( $LineItemPaymentId );
#----------------------------------------------------------------------------------------
# description  Gets a Mollie Status for an OrderId defined in LineItemPayment.
#----------------------------------------------------------------------------------------
# input        LineItemPaymentId | LineItemPayment Id | int
# return       MolliePaymentsClient | object | DE_EPAGES::MolliePayments::API::MolliePaymentsClient
#========================================================================================
sub handleStatusOnPaymentWebhook {
    my $self = shift;
    my ($LineItemPayment, $WebhookPaymentId) = @_;

    if ( not isValid($WebhookPaymentId)) {
        Error("Error validating WebhookPaymentId: \"$WebhookPaymentId\"");
    }

    if (not $LineItemPayment->instanceOf(LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS)) {
        Error('Wrong LineItemPayment class on updateStatusViaPaymentWebhook');
    }

    my $LineItemPaymentStatus = $LineItemPayment->get('TransStatus');

    if (isValid($LineItemPaymentStatus) && ($LineItemPaymentStatus ~~ DEFINITIVE_MOLLIE_PAYMENT_STATUS)) {
        Error(sprintf('LineItemPaymentID: %s already has a definitive status: %s',
            $LineItemPayment->id(),
            $LineItemPaymentStatus)
        );
    }

    my $hMollieOrder = $self->_getMollieOrderById($LineItemPayment);
    my $hMolliePaymentData = $hMollieOrder->{'payments'}->[0];

    if ($hMolliePaymentData->{'id'} ne $WebhookPaymentId) {
        Error(sprintf('The WebhookPaymentId: %s does not match the one on LineItemPaymentID: %s',
            $WebhookPaymentId,
            $LineItemPayment->id())
        );
    }

    $LineItemPayment->set({
        'TransID' => $WebhookPaymentId
    });

    return $self->handleLineItemPaymentStatusChange($LineItemPayment, $hMollieOrder);

}

#========================================================================================
# function     getMollieOrderAndHandleStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentStatusService->getMollieOrderAndHandleStatus( $LineItemPayment );
#----------------------------------------------------------------------------------------
# description  Gets Mollie Order information and updates the status and refunds info received.
#----------------------------------------------------------------------------------------
# input        LineItemPayment | LineItemPayment | object
# return       updated Status | String | String
#========================================================================================
sub getMollieOrderAndHandleStatus {
    my $self = shift;
    my ($LineItemPayment) = @_;

    if (not $LineItemPayment->instanceOf(LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS)) {
        Error('Error supplied payment is not a MolliePayments object');
    }

    my $MollieOrderId = $LineItemPayment->get('MollieOrderId');
    if (not isValid($MollieOrderId)) {
        Error('Invalid Mollie order id');
    }

    my $hMollieOrder = $self->_getMollieOrderById($LineItemPayment);
    my $hMolliePaymentData = $hMollieOrder->{'payments'}->[0];

    $LineItemPayment->set({
        'TransID' => $hMolliePaymentData->{'id'}
    });

    return $self->handleLineItemPaymentStatusChange($LineItemPayment, $hMollieOrder);
}

#----------------------------------------------------------------------------------------
# Private methods
#----------------------------------------------------------------------------------------
sub _cancelMollieOrderById {
    my $self = shift;
    my ($LineItemPayment)  = @_;

    my $MollieOrderId = $LineItemPayment->get('MollieOrderId');
    my $PaymentMethod = $LineItemPayment->get('PaymentMethod');

    if (( not defined($PaymentMethod) ) ||
        ( not $PaymentMethod->instanceOf(PAYMENT_METHOD_MOLLIE_PAYMENTS))) {
        Error('Error supplied payment method is not a MolliePayments object');
    }

    my $OrdersRequest = $self->_createOrdersRequest({'payment' => $PaymentMethod});

    $OrdersRequest->withOrderId($MollieOrderId);
    my $MollieService = $self->_createMollieService($OrdersRequest);

    my $hResponse = $MollieService->withTestMode($LineItemPayment->get('IsTestTransaction'))->doDeleteRequest();

    $self->_LogPayment(
            'Deleted Mollie Order', 
            {
                'MollieOrderId' => $MollieOrderId,
                'Response' => $hResponse
            });

    return $hResponse;
}

sub _getMollieOrderById {
    my $self = shift;
    my ($LineItemPayment)  = @_;

    my $MollieOrderId = $LineItemPayment->get('MollieOrderId');
    my $PaymentMethod = $LineItemPayment->get('PaymentMethod');

    if (( not defined($PaymentMethod) ) ||
        ( not $PaymentMethod->instanceOf(PAYMENT_METHOD_MOLLIE_PAYMENTS))) {
            Error('Error supplied payment method is not a MolliePayments object');
    }

    my $OrdersRequest = $self->_createOrdersRequest({'payment' => $PaymentMethod});
    $OrdersRequest->withEmbed(ORDER_PARAM_EMBED_PAYMENTS)->withOrderId($MollieOrderId);

    my $MollieService = $self->_createMollieService($OrdersRequest);

    my $hResponse = $MollieService->withTestMode($LineItemPayment->get('IsTestTransaction'))->doGetRequest();

    $self->_LogPayment(
        'Get Mollie Order by Id Response', 
        {
           'MollieOrderId' => $MollieOrderId,
           'Response' => $hResponse,
        }
    );

    if (( not isValid($hResponse->{'payments'})) || ( not isValid($hResponse->{'payments'}->[0]))) {
        Error('Invalid order payment. Payment not found.');
    }

    return $hResponse;
}

sub handleLineItemPaymentStatusChange {
    my $self = shift;
    my ($LineItemPayment, $hMollieOrder) = @_;

    my $hMolliePaymentData = $hMollieOrder->{'payments'}->[0];
    my $NewPaymentStatus = $hMolliePaymentData->{'status'};

    my $CurrentPaymentStatus = $LineItemPayment->get('TransStatus');

    if (not ($NewPaymentStatus ~~ SUPPORTED_MOLLIE_PAYMENT_STATUS)) {
        $self->_LogPayment(
            'Unknown Mollie Payment Status Received', 
            {
                'NewStatus' => $NewPaymentStatus,
                'CurrentPaymentStatus' => $CurrentPaymentStatus,
                'LineItemPayment' => LineItemPayment->guid()
            });
        return $CurrentPaymentStatus;
    }

    if (isValid($CurrentPaymentStatus) && ($CurrentPaymentStatus ~~ DEFINITIVE_MOLLIE_PAYMENT_STATUS)) {
        return $CurrentPaymentStatus;
    }

    my $MollieOrderStatus = $hMollieOrder->{'status'};
    if (($MollieOrderStatus ne MOLLIE_ORDER_STATUS_CANCELED) &&
        ($NewPaymentStatus ~~ MOLLIE_PAYMENT_STATUS_ERRORS ))
    {
        $self->_cancelMollieOrderById($LineItemPayment);
    }

    # Given an ePages Basket that has an expiration time larger than the Mollie payment
    # When the shopper wants to order this Basket with an expired status.
    # Then we clean our Mollie info to allow to create a new Mollie order.
    my $BasketOrOrder = $LineItemPayment->container()->parent();
    if ( ($NewPaymentStatus eq PAYMENT_STATUS_EXPIRED) && ($BasketOrOrder->instanceOf('Basket'))) {
        $self->_LogPayment(
            'PaymentStatusService::handleLineItemPaymentStatusChange Received expired status on a Basket. Expired status will not get set.',
            {
                'OldStatus' => $CurrentPaymentStatus,
                'NewStatus' => $NewPaymentStatus,
            }
        );
        $self->deleteLineItemPaymentStatusInfo($LineItemPayment);
        return $NewPaymentStatus;
    }

    $self->_updateLineItemPaymentWithStatusAndRefunds($LineItemPayment, $NewPaymentStatus, $hMolliePaymentData);

    $self->_LogPayment(
            'PaymentStatusService::handleLineItemPaymentStatusChange Updated mollie payment status',
            {
                'OldStatus' => $CurrentPaymentStatus,
                'NewStatus' => $NewPaymentStatus,
            }
    );

    return $NewPaymentStatus;
}

sub deleteLineItemPaymentStatusInfo {
    my $self = shift;
    my ($LineItemPayment) = @_;

    $LineItemPayment->set({
        'TransStatus' => undef,
        'TransID' => undef,
        'MollieOrderId' => undef,
    });

    $self->_LogPayment(
        'PaymentStatusService::deleteLineItemPaymentStatusInfo Cleaned the LineItemPayment Mollie Info',
        {
            'TransStatus' => undef,
            'TransID' => undef,
            'MollieOrderId' => undef,
        }
    );

    return;
}

sub _updateLineItemPaymentWithStatusAndRefunds {
    my $self = shift;
    my ($LineItemPayment, $NewStatus, $hMolliePaymentData) = @_;

    my $CurrentDateTime = $self->_getCurrentDateTime();
    $LineItemPayment->set({ 'TransTime' => $CurrentDateTime });

    $LineItemPayment->set({'TransStatus' => $NewStatus});

    if ( $NewStatus eq PAYMENT_STATUS_PAID ) {
        my $BasketOrOrder = $LineItemPayment->container()->parent();
        $self->setOrderStatusPaidTime($BasketOrOrder, $CurrentDateTime);
        $self->_updateLineItemRefunds($LineItemPayment, $hMolliePaymentData);
    }
    return;
}

sub setOrderStatusPaidTime {
    my $self = shift;
    my ($Order, $Datetime) = @_;

    return unless $Order->instanceOf('CustomerOrder');

    $Order->set({'PaidOn' => $Datetime});
    return;
}

sub _updateLineItemRefunds {
    my $self = shift;
    my ($LineItemPayment, $hMolliePaymentData) = @_;

    $self->_LogPayment(
            'With Status paid. Updating refund amounts',
            {'MolliePaymentData' => $hMolliePaymentData}
        );
   my $MolliePaymentsRefundsService = $self->_createRefundsService();
   $MolliePaymentsRefundsService->updateLineItemRefundAmounts($LineItemPayment, $hMolliePaymentData);
   return;
}
#----------------------------------------------------------------------------------------
# helper methods
#----------------------------------------------------------------------------------------
sub _createOrdersRequest {
    my $self = shift;
    my ($hParams) = @_;

    return DE_EPAGES::MolliePayments::API::Requests::Orders->new($hParams);
}
#----------------------------------------------------------------------------------------
sub _createMollieService {
    my $self = shift;
    my ($RequestObject) = @_;

    return DE_EPAGES::MolliePayments::API::MolliePaymentsClient->new($RequestObject);
}
#----------------------------------------------------------------------------------------
sub _createRefundsService() {
    my $self = shift;

    my $MolliePaymentsRefundsService = DE_EPAGES::MolliePayments::API::MolliePaymentsRefundsService->new();
    return $MolliePaymentsRefundsService;
}
#----------------------------------------------------------------------------------------
sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, $Name, $DebugData, $Section, $Filter);
}
#----------------------------------------------------------------------------------------
sub _getCurrentDateTime {
    my $self = shift;

    return GetCurrentDateTime();
}

1;

