#========================================================================================
# package      DE_EPAGES::MolliePayments::API::Requests::BaseRequest
# state        public
# description  Helper module for generating MolliePayments requests to epagesj
#========================================================================================
package DE_EPAGES::MolliePayments::API::Requests::BaseRequest;

use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Core::API::Error qw (Error);

use URI;
use URI::QueryParam;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::MolliePayments::API::Requests::BaseRequest->new( );
#----------------------------------------------------------------------------------------
# description  MolliePayments requests to epagesj helper constructor
#----------------------------------------------------------------------------------------
# input        $hData | required data to generate requests to epagesj | hash.ref
# return       $Object | new object | DE_EPAGES::MolliePayments::API::Requests::BaseRequest
#========================================================================================
sub new {
    my $class = shift;
    my ($hData) = @_;

    TestObject(
        'PaymentMethodMolliePayments',
        $hData->{'payment'},
        'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments'
    );

    my $self = bless {
        'data'        => $hData,
        'payload'     => {},
        'queryParams' => {},
        'urlPath'     => {}
    }, $class;

    return $self;
}

#========================================================================================
# function     getEpagesjRequestSuffixUrlPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Url = $ProfilesRequest->getEpagesjRequestSuffixUrlPath( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns the request postfix url path from the epagesj endpoint.
#               Must be implemented in child class.
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $Url | Suffix url path from the epagesj endpoint | string
#========================================================================================
sub getEpagesjRequestSuffixUrlPath {
    Error('getEpagesjRequestSuffixUrlPath is not implemented');
}

#========================================================================================
# function     getHttpHeaders
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestHeader = $self->getHttpHeaders( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns required headers of the request.
#               Must be implemented in child class.
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $hRequestHeader | Request headers hash | hash.ref
#========================================================================================
sub getHttpHeaders {
    Error('getHttpHeaders is not implemented');
}

sub getPayload() {
    Error('getPayload is not implemented');
}

#========================================================================================
# function     getPaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod = $self->getPaymentMethod();
#----------------------------------------------------------------------------------------
# description  It returns the PaymentMethodMolliePayments object from data
#----------------------------------------------------------------------------------------
# return       $PaymentMethodMolliePayments | PaymentMethodMolliePayments object | object
#========================================================================================
sub getPaymentMethod {
    my $self = shift;
    return $self->{'data'}->{'payment'};
}


#========================================================================================
# function     getQueryParams
# state        public
#----------------------------------------------------------------------------------------
# syntax       $QueryParams = $self->getQueryParams();
#----------------------------------------------------------------------------------------
# description  It returns the query Params of the Methods request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $QueryParams | Methods request $QueryParams string | string
#========================================================================================
sub getQueryParams {
    Error('getQueryParams is not implemented');
}

# set testmode param
sub withTestMode {
    my $self = shift;
    my ($IsTestMode) = @_;
    $self->{'queryParams'}->{'testmode'} = $IsTestMode;

    return $self->{'queryParams'};

}

#========================================================================================
# function     getShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Shop = $self->getShop();
#----------------------------------------------------------------------------------------
# description  It returns the shop object from state URL argument
#----------------------------------------------------------------------------------------
# return       $Shop | shop object | object
#========================================================================================
sub getShop {
    my $self = shift;
    return $self->{'data'}->{'payment'}->getSite();
}

#========================================================================================
# function     handleResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hFilteredResponse = $self->handleResponse($hResponse);
#----------------------------------------------------------------------------------------
# description  It returns the response filtered
#               Must be implemented in child class
#----------------------------------------------------------------------------------------
# input        $hResponse | response hash | hash.ref
# return       $hFilteredResponse | filtered response hash | hash.ref
#========================================================================================
#========================================================================================
# function     handleResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hFilteredResponse = $self->handleResponse($hResponse);
#----------------------------------------------------------------------------------------
# description  It returns the response filtered
#----------------------------------------------------------------------------------------
# input        $hResponse | response hash | hash.ref
# return       $hFilteredResponse | filtered response hash | hash.ref
#========================================================================================
sub handleResponse {
    my $self = shift;
    my ($hResponse) = @_;

    return $hResponse;
}

1;

