#========================================================================================
# package      DE_EPAGES::MolliePayments::API::Requests::OrdersLines
# state        public
# description  Helper module for generating MolliePayments orders lines requests to epagesj
#========================================================================================
package DE_EPAGES::MolliePayments::API::Requests::OrdersLines;
use base DE_EPAGES::MolliePayments::API::Requests::Orders;

use DE_EPAGES::Core::API::Error qw (Error);

use strict;

use constant PUT_HTTP_METHOD => 'PUT';
use constant DELETE_HTTP_METHOD => 'DELETE';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::MolliePayments::API::Requests::OrdersLines->new( );
#----------------------------------------------------------------------------------------
# description  MolliePayments orders lines requests to epagesj helper constructor
#----------------------------------------------------------------------------------------
# input        $hOrdersData | required data to generate orders lines requests to epagesj | hash.ref
# return       $Object | new object | DE_EPAGES::MolliePayments::API::Requests::OrdersLines
#========================================================================================
sub new {
    my $class = shift;
    my ($hOrdersData) = @_;

    my $self = $class->SUPER::new($hOrdersData);

    return $self;
}

#========================================================================================
# function     getEpagesjRequestSuffixUrlPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Url = $OrdersLinesRequest->getEpagesjRequestSuffixUrlPath( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns the orders lines postfix url path from the epagesj endpoint
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $Url | orders lines suffix url path from the epagesj endpoint | string
#========================================================================================
sub getEpagesjRequestSuffixUrlPath {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    my $LineItemPayment = $self->{'data'}->{'container'}->get('Payment');
    my $MollieOrderId = $LineItemPayment->get('MollieOrderId');

    if ($HttpRequestMethod eq PUT_HTTP_METHOD) {
        return '/orders/' . $MollieOrderId . '/lines/' . $self->{'urlPath'}->{'orderLineItemId'};
    } elsif ($HttpRequestMethod eq DELETE_HTTP_METHOD) {
        return '/orders/' . $MollieOrderId . '/lines';
    }
}

#========================================================================================
# function     getHttpHeaders
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestHeader = $self->getHttpHeaders( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns required headers of the orders lines request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $hRequestHeader | orders lines request headers hash | hash.ref
#========================================================================================
sub getHttpHeaders {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq PUT_HTTP_METHOD || $HttpRequestMethod eq DELETE_HTTP_METHOD) {
        return {
            'X-AccessToken' => $self->{'data'}->{'payment'}->get('AccessToken')
        };
    }
}

#========================================================================================
# function     getQueryParams
# state        public
#----------------------------------------------------------------------------------------
# syntax       $QueryParams = $self->getQueryParams();
#----------------------------------------------------------------------------------------
# description  It returns the query Params of the Methods request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $QueryParams | Methods request $QueryParams string | string
#========================================================================================
sub getQueryParams {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq DELETE_HTTP_METHOD) {
        return $self->{'queryParams'};
    }

    Error('getQueryParams is not implemented for:' . $HttpRequestMethod);
}

#========================================================================================
# function     getPayload
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hPayload = $self->getPayload( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns the body payload of the orders lines request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $hPayload | order lines request payload hash | hash.ref
#========================================================================================
sub getPayload {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq PUT_HTTP_METHOD) {
        my $LineItemPayment = $self->{'data'}->{'container'}->get('Payment');

        $self->{'payload'}->{'testmode'} = $LineItemPayment->get('IsTestTransaction') ? 'true' : 'false';
        $self->{'payload'}->{'name'} = $self->{'orderLineItem'}->{'name'};
        $self->{'payload'}->{'quantity'} = $self->{'orderLineItem'}->{'quantity'};
        $self->{'payload'}->{'unitPrice'} = $self->{'orderLineItem'}->{'unitPrice'};
        $self->{'payload'}->{'totalAmount'} = $self->{'orderLineItem'}->{'totalAmount'};
        $self->{'payload'}->{'vatRate'} = $self->{'orderLineItem'}->{'vatRate'};
        $self->{'payload'}->{'vatAmount'} = $self->{'orderLineItem'}->{'vatAmount'};
    }

    return $self->{'payload'};
}

#========================================================================================
# function     withOrderLineItem
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->withOrderLineItem( $hLineItem );
#----------------------------------------------------------------------------------------
# description  It adds to the object required order line information for patch (put)
#               an order line
#----------------------------------------------------------------------------------------
# input        $hLineItem | order line item data hash | hash.ref
#========================================================================================
sub withOrderLineItem {
    my $self = shift;
    my ($LineItem, $MollieOrderLineItemId) = @_;

    $self->{'orderLineItem'} = $self->SUPER::getLineItemRequest(
        $LineItem,
        $self->{'data'}->{'container'}->get([ qw( CurrencyID LanguageID )])
    );

    $self->{'urlPath'}->{'orderLineItemId'} = $MollieOrderLineItemId;
}

#========================================================================================
# function     withOrderLineItemIdList
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->withOrderLineItemIdList( $aLineItemIDs );
#----------------------------------------------------------------------------------------
# description  It adds to the object required order line information for cancelling
#               a list of order lines
#----------------------------------------------------------------------------------------
# input        $aLineItemIDs | order line item ids array | array.ref
#========================================================================================
sub withOrderLineItemIdList {
    my $self = shift;
    my ($aLineItemIDs) = @_;

    $self->{'queryParams'}->{'orderLines'} = join(',', @$aLineItemIDs);
}


#========================================================================================
# function     handleResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hFilteredResponse = $self->handleResponse($hResponse);
#----------------------------------------------------------------------------------------
# description  It returns the response filtered
#----------------------------------------------------------------------------------------
# input        $hResponse | response hash | hash.ref
# return       $hMollieOrder | filtered response hash | hash.ref
#========================================================================================
sub handleResponse {
    my $self = shift;
    my ($hResponse) = @_;

    if (ref($hResponse) eq 'HASH') {
        return $hResponse;
    } else {
        return {};
    }
}

1;