#========================================================================================
# package      DE_EPAGES::MolliePayments::API::Requests::RequestWrapper
# state        public
# description  Wrapper for MolliePayments requests with token refresh if expired
#========================================================================================
package DE_EPAGES::MolliePayments::API::Requests::RequestWrapper;
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::MolliePayments::API::Tools qw ( ParseErrorStringToHash );
use DE_EPAGES::MolliePayments::API::Requests::Tokens;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::MolliePayments::API::MolliePaymentsClient;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::EPagesJ::API::Object::RestClient;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::MolliePayments::API::Constants qw (
    PAYMENT_TYPE_MOLLIE_PAYMENTS
    PAYMENT_METHOD_MOLLIE_PAYMENTS
);

use strict;

use constant BASE_URI =>  'payments/mollie/v2';
use constant POST_METHOD => 'POST';
use constant PUT_METHOD => 'PUT';
use constant GET_METHOD => 'GET';
use constant DELETE_METHOD => 'DELETE';
use constant HTTP_STATUS_UNAUTHORIZED => 401;
use constant REQUEST_FAILED_CODE => 'REQUEST_FAILED';

sub new {
    my $class = shift;
    my ($Request)  = @_;

    my $self  = bless {
        'request' => $Request,
        'payment' => $Request->getPaymentMethod(),
        'shop' => $Request->getShop()
    }, $class;

    return $self;
}


sub doGet{
    my $self = shift;
    my ($TestMode) = @_;

    my $Request = $self->{'request'};
    my $EpjEndpoint = BASE_URI.$Request->getEpagesjRequestSuffixUrlPath(GET_METHOD);
    my $Headers = $Request->getHttpHeaders(GET_METHOD);
    my $hQueryParams = $Request->getQueryParams(GET_METHOD);
    if (isValid($TestMode) && $TestMode) {
        $hQueryParams->{'testmode'} = 'true';
    }

    my $DecodedResponse;
    eval {
        $DecodedResponse =  $self->_doRequest(GET_METHOD, $EpjEndpoint, $Headers, $hQueryParams);
    };
    if (ExistsError()) {
        my $Error = $self->_GetError();
        if($self->_isTokenExpired($Error)) {
            $self->_LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, 'Authentication Error', $Error->message());
            $self->_refreshPaymentAccessToken();
            $self->_updatePaymentMethodsAccessToken();
            $Headers->{'X-AccessToken'} = $self->{'payment'}->get('AccessToken');
            return $DecodedResponse =  $self->_doRequest(GET_METHOD, $EpjEndpoint, $Headers, $hQueryParams);
        }
        Error($Error);
    }
    return $DecodedResponse;
}

sub doDelete{
    my $self = shift;
    my ($TestMode) = @_;

    my $Request = $self->{'request'};
    my $EpjEndpoint = BASE_URI.$Request->getEpagesjRequestSuffixUrlPath(DELETE_METHOD);
    my $Headers = $Request->getHttpHeaders(DELETE_METHOD);
    my $hQueryParams = $Request->getQueryParams(DELETE_METHOD);
    if (isValid($TestMode) && $TestMode) {
        $hQueryParams->{'testmode'} = 'true';
    }

    my $DecodedResponse;
    eval {
        $DecodedResponse =  $self->_doRequest(DELETE_METHOD, $EpjEndpoint, $Headers, $hQueryParams);
    };
    if (ExistsError()) {
        my $Error = $self->_GetError();
        if($self->_isTokenExpired($Error)) {
            $self->_LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, 'Authentication Error', $Error->message());
            $self->_refreshPaymentAccessToken();
            $self->_updatePaymentMethodsAccessToken();
            $Headers->{'X-AccessToken'} = $self->{'payment'}->get('AccessToken');
            return $DecodedResponse =  $self->_doRequest(DELETE_METHOD, $EpjEndpoint, $Headers, $hQueryParams);
        }
        Error($Error);
    }
    return $DecodedResponse;
}

sub doPost{
    my $self = shift;
    my ($TestMode) = @_;

    my $Request = $self->{'request'};
    my $EpjEndpoint = BASE_URI.$Request->getEpagesjRequestSuffixUrlPath(POST_METHOD);
    my $Headers = $Request->getHttpHeaders(POST_METHOD);
    my $hRequestPayload = $Request->getPayload(POST_METHOD);
    if (isValid($TestMode) && $TestMode) {
        $hRequestPayload->{'testmode'} = 'true';
    }

    my $DecodedResponse;
    eval {
        $DecodedResponse =  $self->_doRequest(POST_METHOD, $EpjEndpoint, $Headers, $hRequestPayload);
    };
    if (ExistsError()) {
        my $Error = $self->_GetError();
        if($self->_isTokenExpired($Error)) {
            $self->_LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, 'Authentication Error', $Error->message());
            $self->_refreshPaymentAccessToken();
            $self->_updatePaymentMethodsAccessToken();
            $Headers->{'X-AccessToken'} = $self->{'payment'}->get('AccessToken');
            return $DecodedResponse =  $self->_doRequest(POST_METHOD, $EpjEndpoint, $Headers, $hRequestPayload);
        }
        Error($Error);
    }
    return $DecodedResponse;
}

sub doPut{
    my $self = shift;
    my ($TestMode) = @_;

    my $Request = $self->{'request'};
    my $EpjEndpoint = BASE_URI.$Request->getEpagesjRequestSuffixUrlPath(PUT_METHOD);
    my $Headers = $Request->getHttpHeaders(PUT_METHOD);
    my $hRequestPayload = $Request->getPayload(PUT_METHOD);
    if (isValid($TestMode) && $TestMode) {
        $hRequestPayload->{'testmode'} = 'true';
    }

    my $DecodedResponse;
    eval {
        $DecodedResponse =  $self->_doRequest(PUT_METHOD, $EpjEndpoint, $Headers, $hRequestPayload);
    };
    if (ExistsError()) {
        my $Error = $self->_GetError();
        $self->_LogError($Error);
        if($self->_isTokenExpired($Error)) {
            $self->_LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, 'Authentication Error', $Error->message());
            $self->_refreshPaymentAccessToken();
            $self->_updatePaymentMethodsAccessToken();
            $Headers->{'X-AccessToken'} = $self->{'payment'}->get('AccessToken');
            return $DecodedResponse =  $self->_doRequest(PUT_METHOD, $EpjEndpoint, $Headers, $hRequestPayload);
        }
        Error($Error);
    }
    return $DecodedResponse;
}


sub _doRequest {
    my $self = shift;
    my ($HttpMethod, $EpjEndpoint, $hRequestHeader, $hData)  = @_;

    my $Shop = $self->{'shop'};
    my $RestClient = $self->_createEpagesjRestClient($Shop);

    my $Url = $RestClient->{'BaseURL'}.$EpjEndpoint;
    my $RequestUrl = URI::URL->new($Url);

    my $Response;
    if ($HttpMethod eq POST_METHOD) {
        my $JsonRequestPayload = $RestClient->encodeRequest($hData);
        $Response = $RestClient->client()->post($RequestUrl, $hRequestHeader, $JsonRequestPayload);
    } elsif($HttpMethod eq PUT_METHOD) {
        my $JsonRequestPayload = $RestClient->encodeRequest($hData);
        $Response = $RestClient->client()->put($RequestUrl, $hRequestHeader, $JsonRequestPayload);
    } elsif($HttpMethod eq DELETE_METHOD) {
        $RequestUrl->query_form($hData);
        $Response = $RestClient->client()->delete($RequestUrl, $hRequestHeader);
    } else {
        $RequestUrl->query_form($hData);
        $Response = $RestClient->client()->get($RequestUrl, $hRequestHeader);
    }
    return $self->_decodeFromResponse($RestClient, $Response);
}

sub _refreshPaymentAccessToken {
    my $self = shift;

    my $TokenRequest = $self->_buildTokenRequest($self->{'payment'});
    $TokenRequest->withRefreshToken();

    my $EpjEndpoint = BASE_URI . $TokenRequest->getEpagesjRequestSuffixUrlPath(POST_METHOD);
    my $Headers = $TokenRequest->getHttpHeaders(POST_METHOD);
    my $hRequestPayload = $TokenRequest->getPayload(POST_METHOD);

    my $DecodedResponse;
    eval {
        $DecodedResponse =  $self->_doRequest(POST_METHOD, $EpjEndpoint, $Headers, $hRequestPayload);
        if(defined $DecodedResponse && isValid($DecodedResponse->{'access_token'})) {
            $self->{'payment'}->set({ 'AccessToken' => $DecodedResponse->{'access_token'} });
        }
    };
    if (ExistsError()) {
        my $Error = GetError();
        $self->_LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, 'Refresh token request failed.', $Error->message());
        Error($Error);
    }
}

sub _updatePaymentMethodsAccessToken{
    my $self = shift;

    my $AccessToken = $self->{'payment'}->get('AccessToken');
    my $RefreshToken = $self->{'payment'}->get('RefreshToken');

    if(not isValid($AccessToken) or not isValid($RefreshToken) ) {
        return;
    }

    my $PaymentMethods = $self->{'shop'}->folder('PaymentMethods')->children;
    foreach my $CurrentPaymentMethod (@$PaymentMethods) {
        next unless ($CurrentPaymentMethod->instanceOf(PAYMENT_METHOD_MOLLIE_PAYMENTS));

        my $CurrentRefreshToken = $CurrentPaymentMethod->get('RefreshToken');
        if(isValid($CurrentRefreshToken) && ($CurrentRefreshToken eq $RefreshToken) ) {
            $CurrentPaymentMethod->set({ 'AccessToken' => $AccessToken });
            $self->_LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, 'AccessToken updated for Payment Method ',
                $CurrentPaymentMethod->alias() , undef, undef);
        }
    }
}

# -----------------------------------------------------------
sub _createEpagesjRestClient {
    my $self = shift;
    my ($Shop) = @_;

    my $System = LoadRootObject();
    my $hArgs = {};

    $hArgs->{BaseURL} = $System->get('OAuthServiceBaseUrl') // $System->get('EPagesJBaseURL');
    $hArgs->{ShopType} = $Shop->get('ShopTypeRef')->alias();
    $hArgs->{Shop} = $Shop;
    $hArgs->{BusinessUnit} = $System->get('ProviderID');

    return DE_EPAGES::EPagesJ::API::Object::RestClient->create( $System, $hArgs );
}

# -----------------------------------------------------------
sub _isTokenExpired {
    my $self = shift;
    my ($Error)  = @_;

    my $isTokenExpired = 0;
    if (defined($Error) &&
        ($Error->code eq REQUEST_FAILED_CODE)) {
        my $hResponse = $self->_ParseErrorStringToHash(
                $Error->vars->{'Response'}, $Error->vars->{'Status'});

        $isTokenExpired = (defined $hResponse->{'content'} && isValid($hResponse->{'content'}->{'status'}) &&
            $hResponse->{'content'}->{'status'} eq HTTP_STATUS_UNAUTHORIZED );
    }
    return $isTokenExpired;
}

# -----------------------------------------------------------
sub _GetError {
    my $self = shift;

    return GetError();
}
# -----------------------------------------------------------
sub _buildTokenRequest {
    my $self = shift;
    my ($PaymentMethod) = @_;

    my $hTokensData = { 'payment' =>  $PaymentMethod};
    my $TokenRequest = $self->_createTokenRequest($hTokensData);

    return $TokenRequest;
}
# -----------------------------------------------------------
sub _createTokenRequest {
    my $self = shift;
    my ($hTokensData) = @_;

    return DE_EPAGES::MolliePayments::API::Requests::Tokens->new($hTokensData);;
}
# -----------------------------------------------------------
sub _ParseErrorStringToHash {
    my $self = shift;
    my ($ErrorString, $StatusCode) = @_;

    return ParseErrorStringToHash($ErrorString, $StatusCode);
}
# -----------------------------------------------------------
sub _LogPayment {
    my $self = shift;
    my ($PaymentTypeAlias, $Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment($PaymentTypeAlias, $Name, $DebugData, $Section, $Filter);
}

# -----------------------------------------------------------
sub _decodeFromResponse {
    my $self = shift;
    my ($Client, $Response) = @_;

    my $ResponseContent = $Response->decoded_content();
    # Empty content responses fail to decode (i.e. "No Content - 204")
    return $ResponseContent unless (isValid($ResponseContent));

    return $Client->decodeResponse( $ResponseContent );
}

1;