#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::MollieOrderShipmentService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::MolliePayments::API::MollieOrderShipmentService
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::MollieOrderShipmentService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::MolliePayments::API::Constants qw ( PAYMENT_STATUS_AUTHORIZED );

use Test::Mockify;
use Test::Mockify::Matcher qw ( Object );
use Test::Mockify::ReturnValue;
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled );
use Test::More;

use strict;

use constant MOLLIE_ORDER_RESPONSE => {
    'amount' => {
        'currency' => 'EUR',
        'value' => '100.70'
    },
    'id' => 'ord_pkq6hu',
    'lines' => [
        {
            'id' => 'odl_kcbaf6',
            'isCancelable' => 1,
            'metadata' => {
                'alias' => 'LineItemProduct_5EC3834B-F9E5-344D-D19C-AC1732063917'
            },
            'name' => 'T-Shirt - olive',
            'quantity' => 1,
            'status' => 'authorized',
            'totalAmount' => {
                'currency' => 'EUR',
                'value' => '29.90'
            },
            'unitPrice' => {
                'currency' => 'EUR',
                'value' => '29.90'
            },
            'vatAmount' => {
                'currency' => 'EUR',
                'value' => '4.77'
            },
            'vatRate' => '19.00'
        },
        {
            'id' => 'odl_qfkxo4',
            'isCancelable' => 1,
            'metadata' => {
                'alias' => 'LineItemProduct_5EC3835A-3DE0-6643-71EB-AC17320639E1'
            },
            'name' => 'Gym bag - brombeer',
            'quantity' => 2,
            'status' => 'authorized',
            'totalAmount' => {
                'currency' => 'EUR',
                'value' => '63.80'
            },
            'unitPrice' => {
                'currency' => 'EUR',
                'value' => '31.90'
            },
            'vatAmount' => {
                'currency' => 'EUR',
                'value' => '10.19'
            },
            'vatRate' => '19.00'
        },
        {
            'id' => 'odl_wxtsyu',
            'isCancelable' => 1,
            'metadata' => {
                'alias' => 'LineItemShipping_5EC3834B-E6D4-0889-A2AE-AC1732063987'
            },
            'name' => 'Standardzustellung',
            'quantity' => 1,
            'status' => 'authorized',
            'totalAmount' => {
                'currency' => 'EUR',
                'value' => '7.00'
            },
            'unitPrice' => {
                'currency' => 'EUR',
                'value' => '7.00'
            },
            'vatAmount' => {
                'currency' => 'EUR',
                'value' => '1.12'
            },
            'vatRate' => '19.00'
        },
        {
            'id' => 'odl_3ij9lk',
            'isCancelable' => 1,
            'metadata' => {
                'alias' => 'LineItemTax_5EC3834B-C3E4-C6E8-4132-AC17320639C6'
            },
            'name' => 'tax(normal)',
            'quantity' => 1,
            'status' => 'authorized',
            'totalAmount' => {
                'currency' => 'EUR',
                'value' => '0.00'
            },
            'unitPrice' => {
                'currency' => 'EUR',
                'value' => '0.00'
            },
            'vatAmount' => {
                'currency' => 'EUR',
                'value' => '0.00'
            },
            'vatRate' => '19.00'
        }
    ],
    'mode' => 'test',
    'payments' => [
        {
            'amount' => {
                'currency' => 'EUR',
                'value' => '100.70'
            },
            'id' => 'tr_sPTAP9jT4S',
            'orderId' => 'ord_pkq6hu',
            'profileId' => 'pfl_K4a7atmW6J',
            'status' => 'authorized'
        }
    ],
    'status' => 'authorized'
};
use constant MOLLIE_SHIPMENT_RESPONSE => {
    'resource' => 'shipment',
    'id' => 'shp_3wmsgCJN4U',
    'orderId' => 'ord_pkq6hu',
    'lines' => []
};
use constant MOLLIE_RESPONSE_ERROR => {
    'StatusCode' => 'Not Found',
    'Message' => 'Order ord_8sxlle exists, but the wrong mode is used. Try switching live / test API keys.'
};
use constant CAPTURE_OVERFLOW_ERROR => {
    'StatusCode' => 'CaptureOverflow',
    'Message' => 'Order amount (162.5) exceeds authorized amount (100.70)'
};
use constant ORDER_UPDATE_SUCCESS => {
    'StatusCode' => 'OK',
    'Message' => 'Success'
};
use constant ORDER_SHIPPING_SUCCESS => {
    'StatusCode' => 'OK',
    'Message' => 'Shipment Id shp_3wmsgCJN4U created'
};


sub test {
    my $self = shift;

    $self->test_mollie_order_line_items_update_with_order_error();
    $self->test_mollie_order_line_items_update_with_capture_overflow();
    $self->test_mollie_order_line_items_update_no_line_item_changes();
    $self->test_mollie_order_line_items_update_no_line_item_changes();
    $self->test_mollie_order_line_items_update_with_product_line_item_update();
    $self->test_mollie_order_line_items_update_with_product_line_item_deletion();

    $self->test_mollie_order_shipment_with_error();
    $self->test_mollie_order_shipment_success();

    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_mollie_order_line_items_update_with_order_error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hMockedClients = {
        'OrderClient' => $self->_createOrderClient()
    };
    my $MockedCustomerOrder = $self->_createCustomerOrder({ 't-shirt' => 1, 'gym-bag' => 2 });
    my $IsOrderCallError = 1;
    my $SUT = $self->_createSUT($hMockedClients, $MockedCustomerOrder, $IsOrderCallError);

    my $hDialogResponse = $SUT->updateMollieOrderLineItems();

    ok(
        WasCalled($SUT, '_formatErrorResponseForDialog'),
        "$SubTestName - Error dialog error response is being generated."
    );
    is_deeply(
        $hDialogResponse, MOLLIE_RESPONSE_ERROR, "$SubTestName - Mollie order gathering returns error."
    );
}

#----------------------------------------------------------------------------------------
sub test_mollie_order_line_items_update_with_capture_overflow {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hMockedClients = {
        'OrderClient' => $self->_createOrderClient(MOLLIE_ORDER_RESPONSE)
    };
    # Mollie Order holds 1 t-shirt and 2 gym-bag products, so epages order was updated and overflows.
    my $MockedCustomerOrder = $self->_createCustomerOrder({ 't-shirt' => 2, 'gym-bag' => 3 });
    my $IsOrderCallError = 0;
    my $SUT = $self->_createSUT($hMockedClients, $MockedCustomerOrder, $IsOrderCallError);

    my $hDialogResponse = $SUT->updateMollieOrderLineItems();

    is_deeply(
        $hDialogResponse, CAPTURE_OVERFLOW_ERROR, "$SubTestName - Capture overflow (authorized lower than captured)."
    );
}

#----------------------------------------------------------------------------------------
sub test_mollie_order_line_items_update_no_line_item_changes {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hMockedClients = {
        'OrderClient' => $self->_createOrderClient(MOLLIE_ORDER_RESPONSE)
    };
    # Mollie Order holds 1 t-shirt and 2 gym-bag products, so both fit.
    my $MockedCustomerOrder = $self->_createCustomerOrder({ 't-shirt' => 1, 'gym-bag' => 2 });
    my $IsOrderCallError = 0;
    my $SUT = $self->_createSUT($hMockedClients, $MockedCustomerOrder, $IsOrderCallError);

    my $hDialogResponse = $SUT->updateMollieOrderLineItems();

    is_deeply(
        $hDialogResponse, ORDER_UPDATE_SUCCESS, "$SubTestName - Successful update when no line item changes required."
    );
}

#----------------------------------------------------------------------------------------
sub test_mollie_order_line_items_update_with_product_line_item_update {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hMockedClients = {
        'OrderClient'    => $self->_createOrderClient(MOLLIE_ORDER_RESPONSE),
        'LineItemClient' => $self->_createLineItemClient()
    };
    # Mollie Order holds 1 t-shirt and 2 gym-bag products, so epages order has been updated.
    my $MockedCustomerOrder = $self->_createCustomerOrder({ 't-shirt' => 1, 'gym-bag' => 1 });
    my $IsOrderCallError = 0;
    my $SUT = $self->_createSUT($hMockedClients, $MockedCustomerOrder, $IsOrderCallError);

    my $hDialogResponse = $SUT->updateMollieOrderLineItems();

    ok(
        WasCalled($SUT, '_buildClientToPutMollieOrderLineItem'),
        "$SubTestName - Build client to update mollie product line item."
    );
    ok(
        WasCalled($hMockedClients->{'LineItemClient'}, 'doPutRequest'),
        "$SubTestName - Mollie product line item updated"
    );
    ok(
        !WasCalled($SUT, '_buildClientToDeleteMollieOrderLineItems'),
        "$SubTestName - No need to build client to delete mollie product line item."
    );
    is_deeply(
        $hDialogResponse, ORDER_UPDATE_SUCCESS, "$SubTestName - Successful update when product line item change required."
    );
}

#----------------------------------------------------------------------------------------
sub test_mollie_order_line_items_update_with_product_line_item_deletion {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hMockedClients = {
        'OrderClient'    => $self->_createOrderClient(MOLLIE_ORDER_RESPONSE),
        'LineItemClient' => $self->_createLineItemClient()
    };
    # Mollie Order holds 1 t-shirt and 2 gym-bag products, so epages order has been updated.
    my $MockedCustomerOrder = $self->_createCustomerOrder({ 't-shirt' => 0, 'gym-bag' => 2 });
    my $IsOrderCallError = 0;
    my $SUT = $self->_createSUT($hMockedClients, $MockedCustomerOrder, $IsOrderCallError);

    my $hDialogResponse = $SUT->updateMollieOrderLineItems();

    ok(
        !WasCalled($SUT, '_buildClientToPutMollieOrderLineItem'),
        "$SubTestName - No need to build client to update mollie product line item."
    );
    ok(
        WasCalled($SUT, '_buildClientToDeleteMollieOrderLineItems'),
        "$SubTestName - Build client to delete mollie product line item."
    );
    ok(
        WasCalled($hMockedClients->{'LineItemClient'}, 'doDeleteRequest'),
        "$SubTestName - Mollie product line item deleted"
    );
    is_deeply(
        $hDialogResponse,
        ORDER_UPDATE_SUCCESS,
        "$SubTestName - Successful update when product line item deletion required."
    );
}

#----------------------------------------------------------------------------------------
sub test_mollie_order_shipment_with_error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hMockedClients = {
        'OrderShipmentClient' => $self->_createOrderShipmentClient()
    };
    my $MockedCustomerOrder = $self->_createCustomerOrder({ 't-shirt' => 1, 'gym-bag' => 2 });
    my $IsOrderShipmentCallError = 1;
    my $SUT = $self->_createSUT($hMockedClients, $MockedCustomerOrder, $IsOrderShipmentCallError);

    my $hDialogResponse = $SUT->shipMollieOrder();

    ok(
        WasCalled($SUT, '_formatErrorResponseForDialog'),
        "$SubTestName - Dialog error response is being generated."
    );
    is_deeply(
        $hDialogResponse, MOLLIE_RESPONSE_ERROR, "$SubTestName - Mollie order shipment returns error."
    );
}

#----------------------------------------------------------------------------------------
sub test_mollie_order_shipment_success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hMockedClients = {
        'OrderShipmentClient' => $self->_createOrderShipmentClient()
    };
    my $MockedCustomerOrder = $self->_createCustomerOrder({ 't-shirt' => 1, 'gym-bag' => 2 });
    my $IsOrderShipmentCallError = 0;
    my $SUT = $self->_createSUT($hMockedClients, $MockedCustomerOrder, $IsOrderShipmentCallError);

    my $hDialogResponse = $SUT->shipMollieOrder();

    ok(
        WasCalled($SUT, '_updateLineItemPaymentStatus'),
        "$SubTestName - Line item payment status updated."
    );
    is(
        $SUT->{'lineItemPayment'}->get('Amount'),
        100.7,
        "$SubTestName - Line item payment amount correclty set"
    );
    is_deeply(
        $hDialogResponse, ORDER_SHIPPING_SUCCESS, "$SubTestName - Mollie order shipment returns success."
    );
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($hMockedClients, $MockedCustomerOrder, $IsCallError) = @_;

    my $aParameterList = [$MockedCustomerOrder];
    my $MockifySut = Test::Mockify::Sut->new(
        'DE_EPAGES::MolliePayments::API::MollieOrderShipmentService', $aParameterList
    );

    $MockifySut->mockImported('DE_EPAGES::ExternalPayment::API::Log', 'LogPayment')
            ->whenAny()->thenReturnUndef();

    $MockifySut->mockImported('DE_EPAGES::Core::API::Log', 'LogError')
            ->when(Object('DE_EPAGES::Core::API::Error'))->thenReturnUndef();

    $MockifySut->mockImported('DE_EPAGES::Core::API::Error', 'GetError')
            ->when()->thenReturnUndef();

    if ($IsCallError) {
        $MockifySut->mockImported('DE_EPAGES::Core::API::Error', 'ExistsError')
                ->when()->thenReturn(1);

        $MockifySut->mock('_formatErrorResponseForDialog')
                ->whenAny()->thenReturn(MOLLIE_RESPONSE_ERROR);
    }

    if (defined($hMockedClients->{'OrderClient'})) {
        $MockifySut->mock('_buildClientToGetMollieOrder')
                ->when()->thenReturn($hMockedClients->{'OrderClient'});
    }
    if (defined($hMockedClients->{'LineItemClient'})) {
        $MockifySut->mock('_buildClientToPutMollieOrderLineItem')
            ->whenAny()->thenReturn($hMockedClients->{'LineItemClient'});
        $MockifySut->mock('_buildClientToDeleteMollieOrderLineItems')
            ->whenAny()->thenReturn($hMockedClients->{'LineItemClient'});
    }
    if (defined($hMockedClients->{'OrderShipmentClient'})) {
        $MockifySut->mock('_buildClientToShipMollieOrder')
            ->whenAny()->thenReturn($hMockedClients->{'OrderShipmentClient'});
    }

    $MockifySut->mock('_updateLineItemPaymentStatus')
            ->whenAny()->thenReturnUndef();

    return $MockifySut->getMockObject();
}

sub _createOrderClient {
    my $self = shift;
    my ($hMollieOrder) = @_;

    my $MockifyClient = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::MolliePayments::API::MolliePaymentsClient');
    $MockifyClient->addMockWithReturnValue('doGetRequest', $hMollieOrder);

    return $MockifyClient->getMockObject();
}

sub _createLineItemClient {
    my $self = shift;

    my $MockifyClient = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::MolliePayments::API::MolliePaymentsClient');
    $MockifyClient->addMockWithReturnValue('doPutRequest', { });
    $MockifyClient->addMockWithReturnValue('doDeleteRequest', { });

    return $MockifyClient->getMockObject();
}

sub _createOrderShipmentClient {
    my $self = shift;

    my $MockifyClient = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::MolliePayments::API::MolliePaymentsClient');
    $MockifyClient->addMockWithReturnValue('doPostRequest', MOLLIE_SHIPMENT_RESPONSE);

    return $MockifyClient->getMockObject();
}

sub _createCustomerOrder {
    my $self = shift;
    my ($hProductQuantities) = @_;

    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer($hProductQuantities)
    };
    my $MockifyCustomerOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes
    );

    return $MockifyCustomerOrder->getMockObject();
}

sub _createLineItemContainer {
    my $self = shift;
    my ($hProductQuantities) = @_;

    my @ContainerLineItems = ();
    push(@ContainerLineItems, $self->_createLineItemShipping());
    push(@ContainerLineItems, $self->_createLineItemTax());
    if ($hProductQuantities->{'t-shirt'} > 0) {
        push(@ContainerLineItems, $self->_createLineItemProduct1($hProductQuantities->{'t-shirt'}));
    }
    if ($hProductQuantities->{'gym-bag'} > 0) {
        push(@ContainerLineItems, $self->_createLineItemProduct2($hProductQuantities->{'gym-bag'}));
    }

    my $TotalAmount = 0;
    foreach my $LineItem (@ContainerLineItems) {
        $TotalAmount += $LineItem->get('LineItemPrice');
    }

    my $hAttributes = {
        'GrandTotal' => $TotalAmount,
        'Payment' => $self->_createLineItemPayment()
    };
    my $MockifyCointainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes
    );
    $MockifyCointainer->addMockWithReturnValue('children', \@ContainerLineItems);

    return $MockifyCointainer->getMockObject();
}

sub _createLineItemShipping {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'LineItemShipping_5EC3834B-E6D4-0889-A2AE-AC1732063987',
        'BasePrice' => undef,
        'LineItemPrice' => 7,
        'Quantity' => 1
    };
    my $MockifyLineItemShipping = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemShipping', $hAttributes
    );
    $MockifyLineItemShipping->mock('instanceOf', sub {
            shift;
            my ($ClassType) = @_;
            return 0 unless ($ClassType =~ m/LineItemShipping/);
            return 1;
    });

    return $MockifyLineItemShipping->getMockObject();
}

sub _createLineItemPayment {
    my $self = shift;

    my $hAttributes = {
        'Amount' => 100.70,
        'AmountRefunded' => undef,
        'AmountRemaining'=> undef,
        'TransStatus' => PAYMENT_STATUS_AUTHORIZED
    };
    my $MockifyLineItemPayment = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments', $hAttributes
    );

    return $MockifyLineItemPayment->getMockObject();
}

sub _createLineItemTax {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'LineItemTax_5EC3834B-C3E4-C6E8-4132-AC17320639C6',
        'BasePrice' => 100.7,
        'LineItemPrice' => 0,
        'Quantity' => undef
    };
    my $MockifyLineItemTax = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemTax', $hAttributes
    );
    $MockifyLineItemTax->mock('instanceOf', sub {
        shift;
        my ($ClassType) = @_;
        return 0 unless ($ClassType =~ m/LineItemTax/);
        return 1;
    });

    return $MockifyLineItemTax->getMockObject();
}

sub _createLineItemProduct1 {
    my $self = shift;
    my ($Quantity) = @_;

    my $hAttributes = {
        'Alias' => 'LineItemProduct_5EC3834B-F9E5-344D-D19C-AC1732063917',
        'BasePrice' => 29.90,
        'LineItemPrice' => 29.9 * $Quantity,
        'Quantity' => $Quantity
    };
    my $MockifyLineItemProduct = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes
    );
    $MockifyLineItemProduct->mock('instanceOf', sub {
        shift;
        my ($ClassType) = @_;
        return 0 unless ($ClassType =~ m/LineItemProduct/);
        return 1;
    });

    return $MockifyLineItemProduct->getMockObject();
}

sub _createLineItemProduct2 {
    my $self = shift;
    my ($Quantity) = @_;

    my $hAttributes = {
        'Alias' => 'LineItemProduct_5EC3835A-3DE0-6643-71EB-AC17320639E1',
        'BasePrice' => 31.90,
        'LineItemPrice' => 31.9 * $Quantity,
        'Quantity' => $Quantity
    };
    my $MockifyLineItemProduct = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes
    );
    $MockifyLineItemProduct->mock('instanceOf', sub {
        shift;
        my ($ClassType) = @_;
        return 0 unless ($ClassType =~ m/LineItemProduct/);
        return 1;
    });

    return $MockifyLineItemProduct->getMockObject();
}


#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
