#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::UI_JSON
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Provider
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::UI_JSON;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::MolliePayments::API::MolliePaymentsClient;
use DE_EPAGES::MolliePayments::API::Requests::Orders;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Mockify;
use Test::Mockify::Matcher qw ( HashRef Any Object String Number);
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );

use Test::Exception;
use Test::MockObject::Extends;
use Test::More;

use strict;

use constant PROFILE_ID => 'pfl_v9hTwCvYq1';
use constant SHOP_ALIAS => 'MyShop';
use constant LINE_ITEM_PAYMENT_GUID => 'AJBD-IOHH-435-IUGU';
use constant SHOP_ID => '6541';
use constant DEREFERRER_TOKEN => 'DEREFERRER_TOKEN';
use constant EXPECTED_TOKEN_PAYLOAD => sprintf('%s&%s&%s', SHOP_ALIAS, SHOP_ID, LINE_ITEM_PAYMENT_GUID);

sub test {
    my $self = shift;

    $self->test_GetMolliePaymentsMethods();
    $self->test_OrderCreationMolliePayments();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub test_GetMolliePaymentsMethods {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hParams =  {
        'IsInTestMode' => 1,
        'ProfileId' => PROFILE_ID
    };

    my $Servlet = $self->_createServletWithPaymentObject($hParams);

    my $MethodsRequest = $self->_createMethodsHandler();
    my $MollieService = $self->_createService();

    my $SUT = $self->_createSutMethods($MethodsRequest, $MollieService);

    $SUT->GetMolliePaymentsMethods($Servlet);

    is_deeply(  GetParametersFromMockifyCall( $MethodsRequest, 'withProfileId' ),
                [ PROFILE_ID ],
                "$SubTestName - Methods withProfileId was called with correct ProfileId"
    );

    is_deeply(  GetParametersFromMockifyCall( $MollieService, 'withTestMode' ),
                [ 1 ],
                "$SubTestName - MollieService withTestMode was called with correct IsInTestMode"
    );

    ok(WasCalled($MollieService, 'doGetRequest'), "$SubTestName - MollieService doGetRequest was called");

    is_deeply(  GetParametersFromMockifyCall( $SUT, '_writeContentOutput' ),
                [ {'result' => 'result'} ],
                "$SubTestName - MollieService _writeContentOutput was called with correct parameter"
    );

    return;
}

sub test_OrderCreationMolliePayments {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Service = $self->_createOrdersService();
    my $SUT = $self->_createSutOrders($Service);

    my $Servlet = $self->_createServletWithBasketObject();
    $SUT->OrderCreationMolliePayments($Servlet);

    ok(WasCalled($SUT, '_createMolliePaymentsClient'),
            "$SubTestName - Mollie orders service creation method was called"
    );

    ok(WasCalled($Service, 'doPostRequest'), "$SubTestName - Mollie orders service doPostRequest was called");

    ok(WasCalled($SUT, '_createOrdersHandler'), "$SubTestName - Mollie orders _createOrdersHandler was called");

    is_deeply(
        GetParametersFromMockifyCall($SUT, '_createOrdersHandler')->[0],
        {
            'redirectUrl' => 'https://myshop.epages.com',
            'paymentWebhookUrl' => 'https://myshop.epages.com&DereferrerToken=DEREFERRER_TOKEN',
            'basket' => $Servlet->object(),
            'payment' => $Servlet->object()->get('LineItemContainer')->get('Payment')->get('PaymentMethod'),
            'container' => $Servlet->object()->get('LineItemContainer')
        },
        "$SubTestName - Mollie orders was initiated with correct parameters"
    );

    ok(
        GetParametersFromMockifyCall($SUT, '_LogPayment')->[0] =~ /request/,
        "$SubTestName - Mollie orders service request was logged"
    );

    ok(
        GetParametersFromMockifyCall($SUT, '_LogPayment', 1)->[0] =~ /[Ss]uccess/,
        "$SubTestName - Mollie orders service response was logged and successful"
    );
    is_deeply(
        GetParametersFromMockifyCall($SUT, '_LogPayment', 1)->[1],
        { 'checkout' => { 'href' => 'https://www.mollie.com', 'type' => 'text/html' } },
        "$SubTestName - Mollie orders service response includes checkout data"
    );
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createForm {
    my $self = shift;

    my $Mockify = Test::Mockify->new( 'DE_EPAGES::Presentation::API::Form');
    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'ID' => SHOP_ID,
        'Alias' => SHOP_ALIAS
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createServletWithPaymentObject {
    my $self = shift;
    my ($hParams) = @_;

    my $PaymentMethod = $self->_createPaymentMethod();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($PaymentMethod);
    $MockifyServlet->addMockWithReturnValue( 'form', $self->_createForm() );

    $MockifyServlet->addParam( 'IsInTestMode', $hParams->{'IsInTestMode'} );
    $MockifyServlet->addParam( 'ProfileId', $hParams->{'ProfileId'} );

    return $MockifyServlet->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createServletWithBasketObject {
    my $self = shift;
    my ($hParams) = @_;

    my $Basket = $self->_createBasket();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Basket);

    return $MockifyServlet->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createSutMethods {
    my $self = shift;
    my ($MethodsRequest, $MollieService) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::UI::JSON');

    $Mockify->mock('_createMethodsHandler')->when(HashRef())->thenReturn($MethodsRequest);
    $Mockify->mock('_createMolliePaymentsClient')->when(Object('DE_EPAGES::MolliePayments::API::Requests::Methods'))->thenReturn($MollieService);
    $Mockify->mock('_writeContentOutput')->when(HashRef())->thenReturnUndef();
    $Mockify->mock('_LogPayment')->when(String(), HashRef(), Any(), Any())->thenReturnUndef();

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createMethodsHandler {
    my $self = shift;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::Requests::Methods');
    $Mockify->mock( 'withProfileId')->when(String())->thenCall(sub{return shift;});

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createSutOrders {
    my $self = shift;
    my ($Service) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::MolliePayments::UI::JSON');

    $MockifySut->mockImported('DE_EPAGES::Shop::API::Url', 'BuildShopUrl')
            ->when(
                    Object(),
                    HashRef(),
                    HashRef(),
                    Object('DE_EPAGES::Presentation::Servlet::BasePermission')
            )
            ->thenReturn('https://myshop.epages.com');
    $MockifySut->mockStatic('DE_EPAGES::MolliePayments::API::Requests::Orders::new')
            ->when(String(), HashRef())->thenReturn($self->_createOrdersHandler());

    $MockifySut->spy('_createOrdersHandler')->when(HashRef());
    $MockifySut->mock('_createMolliePaymentsClient')
            ->when(Object('DE_EPAGES::MolliePayments::API::Requests::Orders'))
            ->thenReturn($Service);
    $MockifySut->mock('_writeContentOutput')->when(HashRef())->thenReturnUndef();
    $MockifySut->mock('_LogPayment')->when(String(), HashRef())->thenReturnUndef();
    $MockifySut->mock('_GenerateDereferrerToken')->when(String(EXPECTED_TOKEN_PAYLOAD))->thenReturn(DEREFERRER_TOKEN);

    return $MockifySut->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createOrdersHandler {
    my $self = shift;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::Requests::Orders');
    $Mockify->mock('getPayload')->when(String())->thenReturn({});

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createService {
    my $self = shift;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::MolliePaymentsClient');
    $Mockify->mock('doGetRequest')->when()->thenReturn({'result' => 'result'});
    $Mockify->mock('withTestMode')->when(Number())->thenReturn($Mockify->getMockObject());

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createOrdersService {
    my $self = shift;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::MolliePaymentsClient');
    $Mockify->mock('doPostRequest')
            ->when()
            ->thenReturn({
                    'checkout' => {
                        'href' => 'https://www.mollie.com',
                        'type' => 'text/html'
                    }
            });

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        'ID' => 123
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;

    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer(),
        'Shop' => $self->_createShop(),
    };
    my $MockifyBasket = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::Basket', $hAttributes
    );

    return $MockifyBasket->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'Payment' => $self->_createLineItemPaymentMolliePayments()
    };
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes
    );

    return $MockifyContainer->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createLineItemPaymentMolliePayments {
    my $self = shift;

    my $hAttributes = {
        'PaymentMethod' => $self->_createPaymentMethod(),
        'GUID' => LINE_ITEM_PAYMENT_GUID
    };
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments', $hAttributes
    );

    return $MockifyContainer->getMockObject();
}

__PACKAGE__->runScript();
