#========================================================================================
# package      DE_EPAGES::Object::API::SQLStructureImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Imports '*.sql' files and returns a SQL structure object.
# example      my $Importer = DE_EPAGES::Object::API::SQLStructureImportHandler->new;
#               my $SQLStruct = $Importer->getSQLStructure('ebaycategory.sql');
#========================================================================================

package DE_EPAGES::Object::API::SQLStructureImportHandler;

use strict;
use DE_EPAGES::Object::API::Object::SQLStructure;
use DE_EPAGES::Core::API::File qw( GetFileContentLines );

#========================================================================================
# function     new()
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler = DE_EPAGES::XML::API::SemanticStructureImportHandler->new();
#----------------------------------------------------------------------------------------
# description  Creates a new SemanticStructureImportHandler object
#----------------------------------------------------------------------------------------
# return       $Handler | SemanticStructureImportHandler | object
#========================================================================================
sub new {
    my $class = shift;
    my ($SQLType) = @_;

    my $self  = bless {}, $class;

    #set SQL type
    $self->{'SQL-Type'} = $SQLType;

    return $self;
}

#========================================================================================
# function     _importSQL
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Package->_importSQL( $FileName );
# syntax       $Handler = $Package->new;
#               $Handler->_importSQL( $FileName );
#----------------------------------------------------------------------------------------
# description  Imports tables, constarints and indices from an SQL file.
#----------------------------------------------------------------------------------------
# input        $FileName | SQL file name | string
#========================================================================================
sub _importSQL {
    my $self = shift;
    my ($FileName) = @_;

    #placeholder to remember actual TableName
    $self->{'_SQLStruct-ActualTableName'} = '';

    #placeholder to remember actual foreign key
    $self->{'_SQLStruct-ActualForeignKey'} = '';

    #placeholder to remember actual foreign key columns
    $self->{'_SQLStruct-ActualForeignKeyColumns'} = undef;

    #placeholder to remember actual index name
    $self->{'_SQLStruct-ActualIndexName'} = '';

    #placeholder to remember actual index clustered value
    $self->{'_SQLStruct-ActualIndexClustered'} = undef;

    #placeholder to remember actual index unique value
    $self->{'_SQLStruct-ActualIndexUnique'} = undef;

    #search mode is necessary to know what the importHandler is looking for
    $self->{'_SQLStruct-SearchMode'} = 'NORMAL';

    #column counter
    $self->{'_SQLStruct-ColumnCounter'} = undef;

    #result sql structure object
    $self->{'_SQLStruct-SQLStructure'} = DE_EPAGES::Object::API::Object::SQLStructure->new(
        $self->{'SQL-Type'}
    );

    $self->{'_SQLStruct-ActualFilename'} = $FileName;

    #START IMPORT
    my $aFileContent = GetFileContentLines($FileName);

    foreach my $ContentLine (@$aFileContent){

        #look for search mode
        if($self->{'_SQLStruct-SearchMode'} eq 'NORMAL'){
            $self->_lookNextForTablesProceduresViewsAndIndex($ContentLine);
        }
        elsif ($self->{'_SQLStruct-SearchMode'} eq 'TABLE'){
            $self->_lookNextForColumnsAndConstraints($ContentLine);
        }
        elsif ($self->{'_SQLStruct-SearchMode'} eq 'FOREIGN_KEY'){
            $self->_lookNextForReferenceOfForeignKey($ContentLine);
        }
        elsif ($self->{'_SQLStruct-SearchMode'} eq 'INDEX'){
            $self->_lookNextForIndexInfos($ContentLine);
        }
        elsif ($self->{'_SQLStruct-SearchMode'} eq 'VIEW'){
            $self->_lookNextForViewInfos($ContentLine);
        }
        elsif ($self->{'_SQLStruct-SearchMode'} eq 'VIEW_SELECT'){
            $self->_lookNextForViewInfosSelect($ContentLine);
        }
        elsif ($self->{'_SQLStruct-SearchMode'} eq 'VIEW_FROM'){
            $self->_lookNextForViewInfosFrom($ContentLine);
        }
        elsif ($self->{'_SQLStruct-SearchMode'} eq 'VIEW_WHERE'){
            $self->_lookNextForViewInfosWhere($ContentLine);
        }
    }

    delete $self->{'_SQLStruct-ActualFilename'};

    return $self->{'_SQLStruct-SQLStructure'};
}

#========================================================================================
# function     getSQLStructure
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SQLStruct = $self->getSQLStructure($FileName);
#----------------------------------------------------------------------------------------
# description  Returns the SQL structure object of an *.sql file.
#----------------------------------------------------------------------------------------
# input        $FileName | SQL file name | string
# return       $SQLStruct | SQL structure object | object
#========================================================================================
sub getSQLStructure{
    my $self = shift;
    my ($FileName) = @_;

    $self->_importSQL($FileName);

    return $self->{'_SQLStruct-SQLStructure'};
}

#========================================================================================
# function     _lookNextForTablesProceduresViewsAndIndex
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_lookNextForTablesProceduresViewsAndIndex($ContentLine);
#----------------------------------------------------------------------------------------
# description  gets a line of an *.sql file and checks for table or index.
#               fills the SQLStructure object.
#----------------------------------------------------------------------------------------
# input        $ContentLine | line of SQL file | string
#========================================================================================
sub _lookNextForTablesProceduresViewsAndIndex {
    my $self = shift;
    my ($ContentLine) = @_;

    my $SQLStruct = $self->{'_SQLStruct-SQLStructure'};

    my $ContentLine_lc = lc($ContentLine);

    if ($ContentLine_lc =~ /^create table [\w\`]+ \(/){
        #get table name
        my $TableName = $ContentLine_lc;
        $TableName =~ s/^create table ([\w\`]+) \(\s*/$1/;

        #set table name to sql structure
        $SQLStruct->addTable($TableName);

        #remember actual table name
        $self->{'_SQLStruct-ActualTableName'} = $TableName;

        #set search mode
        $self->{'_SQLStruct-SearchMode'} = 'TABLE';

        #reset column counter
        $self->{'_SQLStruct-ColumnCounter'} = 1;
     }
     #check for index defined in one line
     elsif ($ContentLine_lc =~ /^create\s*(unique){0,1}\s*(nonclustered|clustered){0,1}\s+index\s+\w+\s+on\s+\w+\s+\(.+\)/){
        #get index name
        my $IndexName = $ContentLine_lc;
        $IndexName =~ s/^.*?index\s+(\w+)\s*.*\n*/$1/;

        #get UNIQUE
        my $Unique;

        if ($ContentLine_lc =~ /unique/){ $Unique = 1; }
        else{ $Unique = 0; }

        #get clustered
        my $Clustered;

        if ($ContentLine_lc =~ /nonclustered/){ $Clustered = 'NONCLUSTERED'; }
        elsif ($ContentLine_lc =~ /clustered/){ $Clustered = 'CLUSTERED'; }
        else { $Clustered = '_SQLStruct-Default'; }

        #get table name
        my $TableName = $ContentLine_lc;
        $TableName =~ s/^.*?on\s+(\w+)\s+\(.*\n*/$1/;

        my $hColumnDescPairs = {};

        my $ColumnDescPairs = $ContentLine_lc;
        $ColumnDescPairs =~ s/.*?on\s+\w+\s+\((.*?)\)\n*/$1/;

        my @ColumnDescPairs = split(/, */,$ColumnDescPairs);
        foreach my $ColumnDescPair (@ColumnDescPairs){
            my $Desc;
            if ($ColumnDescPair =~ /\bdesc\b/){ $Desc = 'DESC'; }
            elsif ($ColumnDescPair =~ /\basc\b/) { $Desc = 'ASC'; }
            else { $Desc = '_SQLStruct-Default'; }

            my $ColumnName = $ColumnDescPair;
            $ColumnName =~ s/^(\w+)\s+.*\n*/$1/;

            $hColumnDescPairs->{$ColumnName} = $Desc;
        }

        $SQLStruct->addIndex($TableName, $IndexName, $hColumnDescPairs, {'Unique' => $Unique, 'Clustered' => $Clustered});

        #set search mode
        $self->{'_SQLStruct-SearchMode'} = 'INDEX';
     }
     #check for index defined in two lines
     elsif ($ContentLine_lc =~ /^create\s*(unique){0,1}\s*(nonclustered|clustered){0,1}\s+index\s+\w+/){

        #get index name
        my $IndexName = $ContentLine_lc;
        $IndexName =~ s/^.*?index\s+(\w+)\s*\n*/$1/;

        #get UNIQUE
        my $Unique;

        if ($ContentLine_lc =~ /unique/){ $Unique = 1; }
        else{ $Unique = 0; }

        #get clustered
        my $Clustered;

        if ($ContentLine_lc =~ /nonclustered/){ $Clustered = 'NONCLUSTERED'; }
        elsif ($ContentLine_lc =~ /clustered/){ $Clustered = 'CLUSTERED'; }
        else { $Clustered = '_SQLStruct-Default'; }

        #set index name
        $self->{'_SQLStruct-ActualIndexName'} = $IndexName;

        #set index clustered value
        $self->{'_SQLStruct-ActualIndexClustered'} = $Clustered;

        #set index unique value
        $self->{'_SQLStruct-ActualIndexUnique'} = $Unique;

        #set search mode
        $self->{'_SQLStruct-SearchMode'} = 'INDEX';
    }
    #check for index defined in two lines
     elsif ($ContentLine_lc =~ /^create\s+view\s+\w+/){

        #get index name
        my $ViewName = $ContentLine_lc;
        $ViewName =~ s/^.*?view\s+(\w+)\s*\n*/$1/;

        #set view name
        $self->{'_SQLStruct-ActualViewName'} = $ViewName;

        #set search mode
        $self->{'_SQLStruct-SearchMode'} = 'VIEW';
    }
    else{
        #ignore following lines
        return if ($ContentLine_lc =~ /^\-\-\s+\#.*/);
        return if ($ContentLine_lc =~ /^\-\-\s+.*/);
        return if ($ContentLine_lc =~ /^go\n/);
        return if ($ContentLine_lc =~ /^\n/);
        die "\nunexpected line '$ContentLine' in search mode 'NORMAL' in file '" . $self->{'_SQLStruct-ActualFilename'} ."'\n";
    }
    return;
}

#========================================================================================
# function     _lookNextForColumnsAndConstraints
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_lookNextForColumnsAndConstraints($ContentLine);
#----------------------------------------------------------------------------------------
# description  gets a line of an *.sql file and checks for columns and constraints.
#               fills the SQLStructure object.
#               called in search mode 'TABLE'
#----------------------------------------------------------------------------------------
# input        $ContentLine | line of SQL file | string
#========================================================================================
sub _lookNextForColumnsAndConstraints {
    my $self = shift;
    my ($ContentLine) = @_;

    my $SQLStruct = $self->{'_SQLStruct-SQLStructure'};

    #get table name
    my $TableName = $self->{'_SQLStruct-ActualTableName'};

    my $ContentLine_lc = lc($ContentLine);

    #look for primary key
    if ($ContentLine_lc =~ /^\,{0,1}\s+constraint\s+\w+\s+primary\skey\s+\([\w\s,]+?\)/){
        #get primary key name
        my $PrimaryKeyName = $ContentLine_lc;
        $PrimaryKeyName =~ s/^\,{0,1}\s+constraint\s+(\w+)\s+primary\skey\s+\([\w\s,]+?\)\s*/$1/;

        #get column names
        my $ColumnString = $ContentLine_lc;
        $ColumnString =~ s/^\,{0,1}\s+constraint\s+\w+\s+primary\skey\s+\(([\w\s,]+?)\)\s*/$1/;

        my @Columns = split(/, */,$ColumnString);

        #set primary key for table
        $SQLStruct->addPrimaryKey(
            $TableName,
            $PrimaryKeyName,
            \@Columns,
            { 'SortOrder' => '_SQLStruct-Default', 'Clustered' => '_SQLStruct-Default'}
        );
    }
    #check for foreign key
    elsif ($ContentLine_lc =~ /^\,{0,1}\s+constraint\s+\w+\s+foreign\skey\s+\([\w\s,]+?\)/){

        #get foreign key name
        my $ForeignKeyName = $ContentLine_lc;
        $ForeignKeyName =~ s/^\,{0,1}\s+constraint\s+(\w+)\s+foreign\skey\s+\([\w\s,]+?\)\n*/$1/;

        #get column names
        my $ColumnString = $ContentLine_lc;
        $ColumnString =~ s/^\,{0,1}\s+constraint\s+\w+\s+foreign\skey\s+\(([\w\s,]+?)\)\s*/$1/;

        my @Columns = split(/, */,$ColumnString);


        #set actual foreign key
        $self->{'_SQLStruct-ActualForeignKey'} = $ForeignKeyName;

        #set actual foreign key columns
        $self->{'_SQLStruct-ActualForeignKeyColumns'} = \@Columns;

        #set search mode
        $self->{'_SQLStruct-SearchMode'} = 'FOREIGN_KEY';
    }
    #check for column
    elsif ($ContentLine_lc =~ /^\,{0,1} +\w+(>:\(.+?\)){0,1} +\w+ *\w+/){

        #get column name
        my $ColumnName =  $ContentLine_lc;
        $ColumnName =~ s/^\,{0,1} +(\w+).*\s*/$1/;

        #get range
        my $DataType = $ContentLine_lc;
        $DataType =~ s/^,{0,1} +\w+ +(.+?) +.*\s*/$1/;

        #get default
        my $UncertainRest = $ContentLine_lc;
        $UncertainRest =~ s/^,{0,1} +\w+ +\w+ +(.*)\s*/$1/;

        #no lower case version
        my $OriginalUncertainRest = $ContentLine;
        $OriginalUncertainRest =~ s/^,{0,1} +\w+ +\w+ +(.*)\s*/$1/;

        #getnot null
        my $NotNull;
        if ($UncertainRest =~ /not\snull/){
            $NotNull = 1;
        }
        elsif ($UncertainRest =~ /null/){
            $NotNull = 0;
        }
        else{
            $NotNull = '_SQLStruct-Default';
        }

        #get default
        my $Default = '_SQLStruct-Default';
        if ($UncertainRest =~ /default/){
            $Default = $OriginalUncertainRest;
            $Default =~ s/^.*?default\s+\'(.*?)\'.*\s*/$1/;
        }
        my $AutoIncrement = 0;
        $AutoIncrement = 1 if ($UncertainRest =~ /auto_increment/);

        #get column position
        my $Position = $self->{'_SQLStruct-ColumnCounter'};

        $SQLStruct->addTableColumn($TableName, $ColumnName, $DataType, $NotNull, $Default, $AutoIncrement, $Position);

        #increase column counter
        $self->{'_SQLStruct-ColumnCounter'} = $Position + 1;
    }
    #check lock_scheme of search mode 'TABLE'
    elsif($ContentLine_lc =~ /^lock*/){
        my $LockSchemeValue = $ContentLine_lc;
        $LockSchemeValue =~ s/^lock\s//;

        #add lock scheme for table
        $SQLStruct->setLockScheme($TableName,$LockSchemeValue);
    }
    #check for end of search mode 'TABLE'
    elsif($ContentLine_lc =~ /^go/){
        #set search mode
        $self->{'_SQLStruct-SearchMode'} = 'NORMAL';
    }
    elsif($ContentLine_lc =~ /^\)/){
        #do nothing
    }
    elsif($ContentLine_lc =~ /^\s*$/){
        # only whitespace - do nothing
    }
    else {
        die "\nunexpected line '$ContentLine' in search mode 'TABLE' in file '" . $self->{'_SQLStruct-ActualFilename'} ."'\n";
    }
    return;
}

#========================================================================================
# function     _lookNextForReferenceOfForeignKey
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_lookNextForReferenceOfForeignKey($ContentLine);
#----------------------------------------------------------------------------------------
# description  gets a line of an *.sql file and checks for referenced table of foreign key.
#               fills the SQLStructure object.
#               called in search mode 'FOREIGN_KEY'
#----------------------------------------------------------------------------------------
# input        $ContentLine | line of SQL file | string
#========================================================================================
sub _lookNextForReferenceOfForeignKey {
    my $self = shift;
    my ($ContentLine) = @_;

    my $SQLStruct = $self->{'_SQLStruct-SQLStructure'};

    my $ContentLine_lc = lc($ContentLine);

    #check for referenced table
    if ($ContentLine_lc =~ /^\s+references\s+[\w\`]+\s+\([\w\s,]+?\)/){
        #get infos
        my $TableName = $self->{'_SQLStruct-ActualTableName'};
        my $ForeignKeyName = $self->{'_SQLStruct-ActualForeignKey'};
        my @ColumnNames = @{$self->{'_SQLStruct-ActualForeignKeyColumns'}};

        #get name of referenced table
        my $ReferencedTableName = $ContentLine_lc;
        $ReferencedTableName =~ s/^\s+references\s+([\w\`]+)\s+\([\w\s,]+?\)\n*/$1/;

        #get column names
        my $ReferencedColumnNames = $ContentLine_lc;
        $ReferencedColumnNames =~ s/^\s+references\s+[\w\`]+\s+\(([\w\s,]+?)\)\n*/$1/;

        my @ReferencedColumnNames = split(/, */,$ReferencedColumnNames);

        #die if one array has more elements
        die "\nsyntax error: mismatched column pairs in primary key '$ForeignKeyName' in sql file '" .
            $self->{'_SQLStruct-ActualFileName'} . "'\n"
            unless (scalar @ColumnNames == scalar @ReferencedColumnNames);

        #generate
        my $hColumnPairs = {};
        foreach my $Count (scalar @ColumnNames -1){
            $hColumnPairs->{$ColumnNames[$Count]} = $ReferencedColumnNames[$Count];
        }

        #add foreign key
        $SQLStruct->addForeignKey($TableName, $ForeignKeyName, $ReferencedTableName, $hColumnPairs);
    }
    else{
        die "\nunexpected line '$ContentLine' in search mode 'FOREIGN_KEY' in file '" . $self->{'_SQLStruct-ActualFilename'} ."'\n";
    }

    #set search mode
    $self->{'_SQLStruct-SearchMode'} = 'TABLE';
}

#========================================================================================
# function     _lookNextForIndexInfos
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_lookNextForIndexInfos($ContentLine);
#----------------------------------------------------------------------------------------
# description  gets a line of an *.sql file and checks for infos of index.
#               fills the SQLStructure object.
#               called in search mode 'INDEX'
#----------------------------------------------------------------------------------------
# input        $ContentLine | line of SQL file | string
#========================================================================================
sub _lookNextForIndexInfos {
    my $self = shift;
    my ($ContentLine) = @_;

    my $SQLStruct = $self->{'_SQLStruct-SQLStructure'};

    my $ContentLine_lc = lc($ContentLine);

    #check for infos of index
    if ($ContentLine_lc =~ /^\s+on\s+\w+\s+\(.*/){

        #get actual index name
        my $IndexName = $self->{'_SQLStruct-ActualIndexName'};

        #placeholder to remember actual index clustered value
        my $Clustered = $self->{'_SQLStruct-ActualIndexClustered'};

        #placeholder to remember actual index unique value
        my $Unique = $self->{'_SQLStruct-ActualIndexUnique'};

        #get table name
        my $TableName = $ContentLine_lc;
        $TableName =~ s/^\s+on\s+(\w+)\s+\(.*\n*/$1/;

        #get column names and sort order
        my $hColumnDescPairs = {};

        my $ColumnDescPairs = $ContentLine_lc;
        $ColumnDescPairs =~ s/^\s+on\s+\w+\s+\((.*?)\n*/$1/;

        my @ColumnDescPairs = split(/, */,$ColumnDescPairs);
        foreach my $ColumnDescPair (@ColumnDescPairs){
            my $Desc;
            if ($ColumnDescPair =~ /desc/){ $Desc = 'DESC'; }
            elsif ($ColumnDescPair =~ /asc/) { $Desc = 'ASC'; }
            else { $Desc = '_SQLStruct-Default'; }

            my $ColumnName = $ColumnDescPair;
            $ColumnName =~ s/^(\w+)\s+.*\n*/$1/;

            $hColumnDescPairs->{$ColumnName} = $Desc;
        }

        $SQLStruct->addIndex($TableName, $IndexName, $hColumnDescPairs, {'Unique' => $Unique, 'Clustered' => $Clustered});
    }
    elsif ($ContentLine_lc =~ /^go/ or $ContentLine_lc =~ /^\)/){
        #set search mode
        $self->{'_SQLStruct-SearchMode'} = 'NORMAL';
    }
    else{
        die "\nunexpected line '$ContentLine' in search mode 'INDEX' in file '" . $self->{'_SQLStruct-ActualFilename'} ."'\n";
    }
}

#========================================================================================
# function     _lookNextForViewInfos
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_lookNextForViewInfos($ContentLine);
#----------------------------------------------------------------------------------------
# description  gets a line of an *.sql file and checks for infos of index.
#               fills the SQLStructure object.
#               called in search mode 'VIEW'
#----------------------------------------------------------------------------------------
# input        $ContentLine | line of SQL file | string
#========================================================================================
sub _lookNextForViewInfos {
    my $self = shift;
    my ($ContentLine) = @_;

    my $ContentLine_lc = lc($ContentLine);

    #check for infos of index
    if ($ContentLine_lc =~ /^select\s+.+/){

        #get view select expression
        my $ViewSelectExpression = $ContentLine;
        $ViewSelectExpression =~ s/\n/ /g;

        #register view select expression
        $self->{'_SQLStruct-ActualViewSelectExpression'} = $ViewSelectExpression;

        #set search mode to view select
        $self->{'_SQLStruct-SearchMode'} = 'VIEW_SELECT';
    }
    elsif ($ContentLine_lc =~ /^as.*/){
        #do nothing ignore content line
        return;
    }
    else{
        die "\nunexpected line '$ContentLine' in search mode 'VIEW' in file '" . $self->{'_SQLStruct-ActualFilename'} ."'\n";
    }

    return;
}

#========================================================================================
# function     _lookNextForViewInfosSelect
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_lookNextForViewInfosSelect($ContentLine);
#----------------------------------------------------------------------------------------
# description  gets a line of an *.sql file and checks for infos of index.
#               fills the SQLStructure object.
#               called in search mode 'VIEW_SELECT'
#----------------------------------------------------------------------------------------
# input        $ContentLine | line of SQL file | string
#========================================================================================
sub _lookNextForViewInfosSelect {
    my $self = shift;
    my ($ContentLine) = @_;

    my $ContentLine_lc = lc($ContentLine);

    #check for infos of index
    if ($ContentLine_lc =~ /^from\s+.+/){

        #get from expression
        my $ViewFromExpression = $ContentLine;
        $ViewFromExpression =~ s/\n/ /g;

        #register from expression
        $self->{'_SQLStruct-ActualViewFromExpression'} = $ViewFromExpression;

        #set search mode to view from
        $self->{'_SQLStruct-SearchMode'} = 'VIEW_FROM';
    }
    else{
        #get view select expression
        my $ViewSelectExpression = $ContentLine;
        $ViewSelectExpression =~ s/\n/ /g;

        #register view select expression
        $self->{'_SQLStruct-ActualViewSelectExpression'} .= $ViewSelectExpression;
    }

    return;
}

#========================================================================================
# function     _lookNextForViewInfosFrom
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_lookNextForViewInfosFrom($ContentLine);
#----------------------------------------------------------------------------------------
# description  gets a line of an *.sql file and checks for infos of index.
#               fills the SQLStructure object.
#               called in search mode 'VIEW_SELECT'
#----------------------------------------------------------------------------------------
# input        $ContentLine | line of SQL file | string
#========================================================================================
sub _lookNextForViewInfosFrom {
    my $self = shift;
    my ($ContentLine) = @_;

    my $ContentLine_lc = lc($ContentLine);

    #check for infos of index
    if ($ContentLine_lc =~ /^where\s+.+/){

        #get where expression
        my $ViewWhereExpression = $ContentLine;
        $ViewWhereExpression =~ s/\n/ /g;

        #register where expression
        $self->{'_SQLStruct-ActualViewWhereExpression'} = $ViewWhereExpression;

        #set search mode for view where expression
        $self->{'_SQLStruct-SearchMode'} = 'VIEW_WHERE';
    }
    else{
        #get from expression
        my $ViewFromExpression = $ContentLine;
        $ViewFromExpression =~ s/\n/ /g;

        #register from expression
        $self->{'_SQLStruct-ActualViewFromExpression'} .= $ViewFromExpression;
    }

    return;
}

#========================================================================================
# function     _lookNextForViewInfosWhere
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_lookNextForViewInfosWhere($ContentLine);
#----------------------------------------------------------------------------------------
# description  gets a line of an *.sql file and checks for infos of index.
#               fills the SQLStructure object.
#               called in search mode 'VIEW_SELECT'
#----------------------------------------------------------------------------------------
# input        $ContentLine | line of SQL file | string
#========================================================================================
sub _lookNextForViewInfosWhere {
    my $self = shift;
    my ($ContentLine) = @_;

    my $SQLStruct = $self->{'_SQLStruct-SQLStructure'};

    my $ContentLine_lc = lc($ContentLine);

    #check for infos of index
    if ($ContentLine_lc =~ /^go.*/){

        #get expressions
        my $SelectEx = $self->{'_SQLStruct-ActualViewSelectExpression'};
        $SelectEx =~ s/\s+$//;
        $SelectEx =~ s/\s+/ /g;

        my $FromEx = $self->{'_SQLStruct-ActualViewFromExpression'};
        $FromEx =~ s/\s+$//;
        $FromEx =~ s/\s+/ /g;

        my $WhereEx = $self->{'_SQLStruct-ActualViewWhereExpression'};
        $WhereEx =~ s/\s+$//;
        $WhereEx =~ s/\s+/ /g;

        #add view to structure
        $SQLStruct->addView(
            $self->{'_SQLStruct-ActualViewName'},
            $SelectEx,
            $FromEx,
            $WhereEx
        );

        #set search mode to normal
        $self->{'_SQLStruct-SearchMode'} = 'NORMAL';
    }
    else{
        #get where expression
        my $ViewWhereExpression = $ContentLine;
        $ViewWhereExpression =~ s/\n/ /g;

        #register where expression
        $self->{'_SQLStruct-ActualViewWhereExpression'} .= $ViewWhereExpression;
    }

    return;
}

1;
