use strict;

use Getopt::Long;
use Time::HiRes qw (time);

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::DataCache::API::Cache qw( UpdateConnection );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath LoadRootObject );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use DE_EPAGES::Core::API::Warning qw( Warning );
RunScript( Sub => \&Main );

sub Main {
    my ($Help, $StoreName, $Password, $ShopName);

    GetOptions(
        'help'        => \$Help,
        'storename=s' => \$StoreName,
        'password=s'  => \$Password,
        'shopname=s'  => \$ShopName
    );

    usage() if $Help;

    unless ($StoreName) {
        print STDERR "Missing parameter storename\n\n";
        usage();
    }

    my $time = time();

    RunOnStore(
        Store    => $StoreName,
        Password => $Password,
        Sub      => sub {
            # build a mapping table. ShopLocale => DummyAddress
            my $hLocaleMapping = {  
                                    'de_DE' => 'DE',
                                    'de_AT' => 'AT',
                                    'de_CH' => 'CH',
                                    'de_LU' => 'LU',
                                    'de_BE' => 'BE',
                                    'en_US' => 'US',         
                                    'en_GB' => 'GB',
                                    'en_IE' => 'IE',
                                    'en_AU' => 'AU',
                                    'en_CA' => 'CA',
                                    'en_CH' => 'CH',
                                    'en_DK' => 'DK',
                                    'fr_FR' => 'FR',
                                    'fr_CH' => 'CH',
                                    'fr_BE' => 'BE',
                                    'fr_CA' => 'CA',
                                    'es_ES' => 'ES',
                                    'es_EC' => 'EC',
                                    'es_MX' => 'MX',
                                    'es_AR' => 'AR',
                                    'es_CO' => 'CO', 
                                    'es_US' => 'US', 
                                    'es_BO' => 'BO', 
                                    'nl_NL' => 'NL',
                                    'nl_BE' => 'BE',
                                    'pt_PT' => 'PT',
                                    'pt_BR' => 'BR',
                                    'it_IT' => 'IT',
                                    'it_CH' => 'CH',
                                    'cs_CZ' => 'CZ',
                                    'da_DK' => 'DK',
                                    'no_NO' => 'NO',
                                    'sv_SE' => 'SE',
                                    'fi_FI' => 'FI',
                                    'ru_RU' => 'RU',
                                    'ru_UA' => 'UA',
                                };

            my $System = LoadRootObject();
            if (defined $ShopName) {
                my $Shop;
                eval {
                    $Shop = LoadObjectByPath('/Shops/' . $ShopName);
                };
                if (ExistsError()) {
                    die "Shop: $ShopName does not exists!";
                }

                if (defined $Shop) {
                    # set attribute DummyCustomerAddress for Shop...
                    SetDummyCustomerAddress($System, $Shop, $hLocaleMapping);
                }
                else {
                    print STDERR  "Shop $ShopName not found\n\n";
                }
            }
            else {
                foreach my $Shop (@{LoadRootObject->get('Shops')}) {
                    SetDummyCustomerAddress($System, $Shop, $hLocaleMapping);
                }
            }

            # flush outdated cacheinfo
            eval {
                UpdateConnection( GetCurrentDBHandle()->cacheKey );
                SynchronizeCache();
            };
            Warning( GetError() ) if ExistsError();
        }
    );

    printf "needed %.3f seconds\n", time - $time;
}

sub SetDummyCustomerAddress {
    my ($System, $Shop, $hLocaleMapping) = @_;

    my $LocaleID = $Shop->get('LocaleID');
    die "No entry found for LocaleID: $LocaleID " unless exists $hLocaleMapping->{$LocaleID};

    my $DummyCustomerAddressAlias = 'DummyCustomerAddress_'. $hLocaleMapping->{$LocaleID};
    $Shop->set({ 'DummyCustomerAddress' => $System->folder('DummyCustomerAddresses')->child($DummyCustomerAddressAlias) });
}



sub usage {
    print <<END_USAGE;
Description:
    Sets the DummyCustomerAddress for all shops
    If no shop is given, all shops on the store will be used.

Usage:
    \$PERL $0 [options]

Options:
    -storename     store name for the database connection
    -shopname      name of the shop (optional)

Example:
    \$PERL $0 -storename Store
    \$PERL $0 -storename Store -shopname DemoShop
END_USAGE

    exit(0);
}
