#========================================================================================
# package      DE_EPAGES::TaxUS::Hooks::Address
# state        public
# description  set the correct localization for the default TaxArea
#========================================================================================
package DE_EPAGES::TaxUS::Hooks::Address;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadClassByAlias LoadObject );
use DE_EPAGES::Tax::API::Shop qw ( GetTaxMethodsFolder EnableShopTaxMethod );
use constant US_COUNTRYID => 840;

#========================================================================================
# function     OnAfterUpdateAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateAddress( $hParams );
#----------------------------------------------------------------------------------------
# description  OnAfterUpdateAddress is triggered, to set the default TaxArea name
#               and address by using attributes from the shop address
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                   <li>AddressID
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateAddress {
    my ($hParams) = @_;

    my $AddressID     = $hParams->{'AddressID'};
    my $ShopCountryID = US_COUNTRYID;

    # test if the address is the shop address
    my $Address = LoadObject($AddressID);

    my $Shop        = $Address->getSite;
    return unless $Shop->instanceOf('Shop');

    # test if we can switch this shop to TaxUS
    return unless $Shop->get('CanUseTaxUS');

    my $ShopAddress    = $Shop->get('Address');

    # do this only for the shop address
    return unless (defined $ShopAddress and $ShopAddress->id eq $AddressID);
    return unless (defined $Address->get('CountryID') and $Address->get('CountryID') eq $ShopCountryID);

    my $TaxMatrixUS    = $Shop->child('TaxMatrixUS');
    my $DefaultTaxArea = $TaxMatrixUS->get('DefaultTaxArea');
    my $State          = $ShopAddress->get('State');

    # without a state we can not get CountrySubdivisions
    return unless defined $State;

    # set values to names for all languages
    my $aObjectIDs = LoadClassByAlias('CountrySubdivision')->getObjects('ByCountryIDCode', [$ShopCountryID, $State]);
    my $StateName = $aObjectIDs->[0]->get('English');

    my $aLanguageIDs = $Shop->languages;
    foreach my $LanguageID (@$aLanguageIDs) {
        $DefaultTaxArea->set({'Name' => "$StateName ($State)"}, $LanguageID);
    }

    # set the shop address as presence address for the default taxarea
    unless ($DefaultTaxArea->get('PresenceAddress')) {
        $DefaultTaxArea->set({'PresenceAddress' => $ShopAddress});
    }

    # test if we have already a physical presence with the state of shops address
    # delete ph. presence (TaxArea with Address) if needed
    if ($TaxMatrixUS->existsChild($State)) {
        my $DeleteTaxArea = $TaxMatrixUS->child($State);
        $DeleteTaxArea->delete if $DefaultTaxArea->id ne $DeleteTaxArea->id;
    }

    # activate TaxMethod TaxUS on Shop if not is active
    my $ShopTaxMethod;
    my $TaxType       = LoadRootObject()->folder('TaxTypes')->child('TaxUS');
    my $MethodsFolder = GetTaxMethodsFolder($Shop);

    if ($MethodsFolder->existsChild($TaxType->alias)) {
        $ShopTaxMethod = $MethodsFolder->child($TaxType->alias);
    }
    else {
        $ShopTaxMethod = $TaxType->newTaxMethod(
            $Shop, {
                'Alias'  => $TaxType->alias,
                'Parent' => $MethodsFolder,
            });
    }

    # save old TaxMatrix as child object of the shop because we have to reactivate
    # it if we change to an other TaxMethod
    my $OldTaxMatrix = $Shop->get('TaxMatrix');

    my $ClonedMatrix;
    $ClonedMatrix = $OldTaxMatrix->clone({
            'Parent'       => $Shop,
            'CreationDate' => GetCurrentDateTime()
        },1
    ) unless $Shop->existsChild($OldTaxMatrix->alias);
    $Shop->set({'TaxMatrix' => $ClonedMatrix}) if defined $ClonedMatrix ;

    return if $ShopTaxMethod->isActive();
    EnableShopTaxMethod($Shop, $ShopTaxMethod, $TaxMatrixUS);

    return;
}

#========================================================================================
# function     OnBeforeUpdateAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateAddress( $hParams );
#----------------------------------------------------------------------------------------
# description  OnBeforeUpdateAddress is triggered, to set the TaxArea to NULL if
#               Shop Address State was changed
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                   <li>Values
#                   <li>Object
#                   <li>AddressID
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateAddress {
    my ($hParams) = @_;

    my $hValues = $hParams->{'Values'};

    # only if state was changed
    return unless defined $hValues->{'State'};

    my $Address = $hParams->{'Object'};
    my $Shop    = $Address->getSite;

    # test if we can switch this shop to TaxUS
    return unless $Shop->get('CanUseTaxUS');

    my $ShopAddress = $Shop->get('Address');
    my $USCountryID = US_COUNTRYID;

    # do this only for the US shop address
    return unless (defined $ShopAddress and $ShopAddress->id eq $Address->id);
    return unless $Address->get('CountryID') eq $USCountryID;

    my $ExistingState = $ShopAddress->get('State');

    # state not changed, do nothing
    return if $ExistingState eq $hValues->{'State'};

    my $DefaultTaxArea = $Shop->get('TaxMatrix')->get('DefaultTaxArea');
    my $Class          = LoadClassByAlias('LineItemContainer');
    foreach my $ContainerID (@{$Class->findObjects('ByTaxAreaID', [$DefaultTaxArea->id])}) {
        my $Container = LoadObject($ContainerID);
        $Container->set({TaxArea => undef});
        $Container->parent->set({IsAddressOK => 0}) if $Container->parent->instanceOf('Basket');
    }
}

1;
